/*
 * "$Id: font.c,v 1.1 1995/12/31 18:10:10 mike Exp mike $"
 *
 *   MS Windows font routines for OpenGL.
 *
 * Contents:
 *
 *   FontCreateBitmaps() - Load Windows font bitmaps into OpenGL display
 *                         lists.
 *   FontDelete()        - Delete the OpenGL display lists used for a font.
 *   FontPuts()          - Display a string using the specified font.
 *   FontPrintf()        - Display a formatted string using the specified
 *                         font.
 *
 * Revision History:
 *
 *   $Log: font.c,v $
 *   Revision 1.1  1995/12/31  18:10:10  mike
 *   Initial revision
 */

/*
 * Include necessary headers.
 */

#include "font.h"
#include <stdarg.h>


/*
 * Limits...
 */

#define MAX_STRING	1024


/*
 * 'FontCreateBitmaps()' - Load Windows font bitmaps into OpenGL display
 *                         lists.
 *
 * Only glyphs 32-127 are loaded at present.
 */

GLuint
FontCreateBitmaps(HDC   hdc,		/* I - Device Context */
                  char  *typeface,	/* I - Font specification */
                  int   height,		/* I - Font height/size in pixels */
	          int   weight,		/* I - Weight of font (bold, etc) */
	          DWORD italic)		/* I - Text is italic */
{
  GLuint	base;			/* Base display list for font */
  HFONT		font;			/* Windows font ID */

  
  if ((base = glGenLists(96)) == 0)
    return (0);

  if (stricmp(typeface, "symbol") == 0)
    font = CreateFont(height, 0, 0, 0, weight, italic, FALSE, FALSE,
                      SYMBOL_CHARSET, OUT_TT_PRECIS,
                      CLIP_DEFAULT_PRECIS, DRAFT_QUALITY,
                      DEFAULT_PITCH, typeface);
  else
    font = CreateFont(height, 0, 0, 0, weight, italic, FALSE, FALSE,
                      ANSI_CHARSET, OUT_TT_PRECIS,
                      CLIP_DEFAULT_PRECIS, DRAFT_QUALITY,
                      DEFAULT_PITCH, typeface);
  
  SelectObject(hdc, font);
  
  wglUseFontBitmaps(hdc, 32, 96, base);

  return (base);
}


/*
 * 'FontDelete()' - Delete the OpenGL display lists used for a font.
 */

void
FontDelete(GLuint font)	/* I - Font to delete */
{
  if (font == 0)
    return;

  glDeleteLists(font, 96);
}


/*
 * 'FontPuts()' - Display a string using the specified font.
 */

void
FontPuts(GLuint font,	/* I - Font to use */
         char   *s)	/* I - String to display */
{
  if (font == 0)
    return;

  if (s == NULL)
    return;

  glPushAttrib(GL_LIST_BIT);
    glListBase(font - 32);
    glCallLists(strlen(s), GL_UNSIGNED_BYTE, s);
  glPopAttrib();
}



/*
 * 'FontPrintf()' - Display a formatted string using the specified font.
 */

void
FontPrintf(GLuint font,		/* I - Font to use */
           char   *format,	/* I - printf() style format string */
           ...)			/* I - Other arguments as necessary */
{
  va_list	ap;			/* Argument pointer */
  char		s[MAX_STRING + 1];	/* Output string */


  if (format == NULL)
    return;

  va_start(ap, format);
  vsprintf(s, format, ap);
  va_end(ap);

  FontPuts(font, s);
}


/*
 * End of "$Id: font.c,v 1.1 1995/12/31 18:10:10 mike Exp mike $".
 */
