/*
 * "stencilct.c" - A test program demonstrating the use of glStencilFunc()
 *                 and glStencilOp() to cut away the middle of a cube.
 */

#include <GL/glaux.h>

/*
 * These #define constants are provided for compatibility between MS Windows
 * and the rest of the world.
 *
 * CALLBACK and APIENTRY are function modifiers under MS Windows.
 */

#ifndef WIN32
#  define CALLBACK
#  define APIENTRY
#endif /* !WIN32 */


/*
 * 'reshape_scene()' - Change the size of the scene...
 */

void CALLBACK
reshape_scene(GLsizei width,	/* I - Width of the window in pixels */
              GLsizei height)	/* I - Height of the window in pixels */
{
 /*
  * Reset the current viewport and perspective transformation...
  */

  glViewport(0, 0, width, height);

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(22.5, (float)width / (float)height, 0.1, 1000.0);

  glMatrixMode(GL_MODELVIEW);
}


/*
 * 'draw_scene()' - Draw a scene containing a cube with a sphere in front of
 *                  it.
 */

void CALLBACK
draw_scene(void)
{
  static float	red_light[4] = { 1.0, 0.0, 0.0, 1.0 };
  static float	red_pos[4] = { 1.0, 1.0, 1.0, 0.0 };
  static float	blue_light[4] = { 0.0, 0.0, 1.0, 1.0 };
  static float	blue_pos[4] = { -1.0, -1.0, -1.0, 0.0 };


 /*
  * Enable drawing features that we need...
  */

  glEnable(GL_DEPTH_TEST);
  glEnable(GL_STENCIL_TEST);
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  glEnable(GL_LIGHT1);

  glShadeModel(GL_SMOOTH);

 /*
  * Clear the color, depth, and stencil buffers...
  */

  glClearColor(0.0, 0.0, 0.0, 0.0);
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT |
          GL_STENCIL_BUFFER_BIT);

 /*
  * Draw the sphere that will be cutting away parts of the cube...
  */

  glStencilFunc(GL_ALWAYS, 1, 1);
  glStencilOp(GL_REPLACE, GL_REPLACE, GL_REPLACE);

  glPushMatrix();
    glTranslatef(-1.0, 0.0, -20.0);
    auxSolidSphere(1.0);
  glPopMatrix();

 /*
  * Clear the color and depth buffers once again...
  */

  glClearColor(0.0, 0.0, 0.0, 0.0);
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

 /*
  * Draw the cube...
  *
  * We have positioned two lights in this scene.  The first is red and
  * located above, to the right, and behind the viewer.  The second is blue
  * and located below, to the left, and in front of the viewer.
  */

  glStencilFunc(GL_NOTEQUAL, 1, 1);
  glStencilOp(GL_KEEP, GL_KEEP, GL_KEEP);

  glLightfv(GL_LIGHT0, GL_DIFFUSE, red_light);
  glLightfv(GL_LIGHT0, GL_POSITION, red_pos);

  glLightfv(GL_LIGHT1, GL_DIFFUSE, blue_light);
  glLightfv(GL_LIGHT1, GL_POSITION, blue_pos);

  glPushMatrix();
    glTranslatef(1.0, 0.0, -20.0);
    glRotatef(15.0, 0.0, 1.0, 0.0);
    glRotatef(15.0, 0.0, 0.0, 1.0);
    auxSolidCube(2.0);
  glPopMatrix();

  auxSwapBuffers();
}


/*
 * 'main()' - Initialize the window and display the scene until the user presses
 *            the ESCape key.
 */

void main()
{
  auxInitDisplayMode(AUX_RGB | AUX_DOUBLE | AUX_DEPTH | AUX_STENCIL);
  auxInitWindow("Stenciling");

  auxReshapeFunc(reshape_scene);
  auxMainLoop(draw_scene);
}


/*
 * End of "stencilct.c".
 */
