//----------------------------------------------------------------------------
//  Project Owlgl
//  Waite Group Press
//  Copyright  1996. All Rights Reserved.
//
//  SUBSYSTEM:    Owlgl Application
//  FILE:         OwlglApp.cpp
//  AUTHOR:       Richard S. Wright Jr.
//
//  OVERVIEW
//  ~~~~~~~~
//  Source file for implementation of TOwlglApp (TApplication).
//
//----------------------------------------------------------------------------

#include <owl/pch.h>

#include <owl/docmanag.h>
#include <owl/filedoc.h>

#include "OwlglApp.h"
#include "OwlglWnV.h"                        // Definition of client class.
#include "OwlglAbD.h"                        // Definition of about dialog.


//{{TOwlglApp Implementation}}



//{{DOC_VIEW}}
DEFINE_DOC_TEMPLATE_CLASS(TFileDocument, TOwlglWindowView, DocType1);
//{{DOC_VIEW_END}}

//{{DOC_MANAGER}}
DocType1 __dvt1("All Files", "*.*", 0, "txt", dtAutoDelete | dtUpdateDir | dtOverwritePrompt);
//{{DOC_MANAGER_END}}


//
// Build a response table for all messages/commands handled by the application.
//
DEFINE_RESPONSE_TABLE1(TOwlglApp, TApplication)
//{{TOwlglAppRSP_TBL_BEGIN}}
  EV_OWLVIEW(dnCreate, EvNewView),
  EV_OWLVIEW(dnClose,  EvCloseView),
  EV_COMMAND(CM_HELPABOUT, CmHelpAbout),
//{{TOwlglAppRSP_TBL_END}}
END_RESPONSE_TABLE;


//--------------------------------------------------------
// TOwlglApp
// ~~~~~
//
TOwlglApp::TOwlglApp() : TApplication("Owlgl")
{
  SetDocManager(new TDocManager(dmSDI, this));

  // INSERT>> Your constructor code here.
}


TOwlglApp::~TOwlglApp()
{
  // INSERT>> Your destructor code here.
}


//--------------------------------------------------------
// TOwlglApp
// ~~~~~
// Application main window construction & intialization.
//
void TOwlglApp::InitMainWindow()
{
  if (nCmdShow != SW_HIDE)
    nCmdShow = (nCmdShow != SW_SHOWMINNOACTIVE) ? SW_SHOWNORMAL : nCmdShow;

  TSDIDecFrame* frame = new TSDIDecFrame(0, GetName(), 0, false, this);

  // Override the default window style for the main window.
  //
      frame->Attr.Style |= WS_BORDER | WS_CAPTION | WS_CLIPCHILDREN | WS_CLIPSIBLINGS | WS_MAXIMIZEBOX | WS_MINIMIZEBOX | WS_SYSMENU | WS_THICKFRAME | WS_VISIBLE;
    frame->Attr.Style &= ~(WS_CHILD);

  // Assign ICON w/ this application.
  //
  frame->SetIcon(this, IDI_SDIAPPLICATION);
  frame->SetIconSm(this, IDI_SDIAPPLICATION);

  // Menu associated with window and accelerator table associated with table.
  //
  frame->AssignMenu(IDM_SDI);

  // Associate with the accelerator table.
  //
  frame->Attr.AccelTable = IDM_SDI;

  SetMainWindow(frame);

  frame->SetMenuDescr(TMenuDescr(IDM_SDI));

}


//--------------------------------------------------------
// TOwlglApp
// ~~~~~
// Application instance initialization.
//
void TOwlglApp::InitInstance()
{
  TApplication::InitInstance();
  GetDocManager()->CreateAnyDoc(0, dtNewDoc);
}



//--------------------------------------------------------
// TOwlglApp
// ~~~~~
// Response Table handlers:
//
void TOwlglApp::EvNewView(TView& view)
{
  GetMainWindow()->SetClientWindow(view.GetWindow());
  if (!view.IsOK())
    GetMainWindow()->SetClientWindow(0);
  else if (view.GetViewMenu())
    GetMainWindow()->MergeMenu(*view.GetViewMenu());
}


void TOwlglApp::EvCloseView(TView&)
{
  GetMainWindow()->SetClientWindow(0);
  GetMainWindow()->SetCaption("Owlgl");
}


//
// Build a response table for all messages/commands handled by the application.
//
DEFINE_RESPONSE_TABLE1(TSDIDecFrame, TDecoratedFrame)
//{{TSDIDecFrameRSP_TBL_BEGIN}}
  EV_WM_QUERYNEWPALETTE,
  EV_WM_PALETTECHANGED,
//{{TSDIDecFrameRSP_TBL_END}}
END_RESPONSE_TABLE;


//{{TSDIDecFrame Implementation}}


TSDIDecFrame::TSDIDecFrame(TWindow* parent, const char far* title, TWindow* clientWnd, bool trackMenuSelection, TModule* module)
:
  TDecoratedFrame(parent, title, clientWnd, trackMenuSelection, module)
{
  // INSERT>> Your constructor code here.

}


TSDIDecFrame::~TSDIDecFrame()
{
  // INSERT>> Your destructor code here.

}


//--------------------------------------------------------
// TOwlglApp
// ~~~~~~~~~~~
//
// Menu Help About Owlgl command
void TOwlglApp::CmHelpAbout()
{
  // Show the modal dialog.
  //
  TOwlglAboutDlg(GetMainWindow()).Execute();
}


int OwlMain(int , char* [])
{
  TOwlglApp   app;
  return app.Run();
}

// Route WM_QUERYNEWPALETTE to child
bool TSDIDecFrame::EvQueryNewPalette()
	{
	bool result;
	TWindow *pGLWindow;

	// Get the child SDI window
	pGLWindow = GetClientWindow();

	// Send the message
	if(pGLWindow)
		pGLWindow->SendMessage(WM_QUERYNEWPALETTE,0,0);

	return TRUE;
	}

// Route the WM_PALETTECHANGES to child
void TSDIDecFrame::EvPaletteChanged(THandle hWndPalChg)
	{
	TWindow *pGLWindow;

	// Get the child SDI window
	pGLWindow = GetClientWindow();

	// Send the message
	if(pGLWindow)
		pGLWindow->SendMessage(WM_PALETTECHANGED, (UINT)hWndPalChg, 
(UINT)0);
	}

