unit code;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, OleCtrls, WAITEGL;

type
  TForm1 = class(TForm)
    Button1: TButton;
    glAPI: TWaiteGLCtrl;
    procedure Button1Click(Sender: TObject);
    procedure glAPISetupRC(Sender: TObject);
    procedure glAPIRender(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private declarations }
  public
    // Procedure to build all the display lists
    procedure BuildLists;

  end;

var
  Form1: TForm1;

  // Display list identifiers
  floorList, poleList, ringListBase: integer;

implementation

{$R *.DFM}

procedure TForm1.Button1Click(Sender: TObject);
begin
     Close;
end;


procedure TForm1.glAPISetupRC(Sender: TObject);
begin
     glAPI.Enable(glAPI.glDepthTest());
     glAPI.Enable(glAPI.glColorMaterial());
     glAPI.Enable(glAPI.glCullFace());
     glAPI.FrontFace(glAPI.glCcw());

     // Set background color to black
     glAPI.ClearColor(0.0, 0.0, 0.0, 1.0);

     // Reset coordinate system
     glAPI.MatrixMode(glAPI.glProjection());
     glAPI.LoadIdentity();

     // Setup perspective for viewing
     glAPI.gluPerspective(45.0 , width/height ,100,600);

     // Viewing transformation
     glAPI.MatrixMode(glAPI.glModelview());
     glAPI.LoadIdentity();
     glAPI.Translate(0.0, 0.0, -300.0);


     glAPI.Enable(glAPI.glLighting());


     // Fairly shiny spot 1
     glAPI.LightModelv(glAPI.glLightModelAmbient(), 0.5, 0.5, 0.5, 0.5);
     glAPI.Lightf(glAPI.glLight0(), glAPI.glDiffuse(), 0.6, 0.6, 0.6, 0.6);
     glAPI.Lightf(glAPI.glLight0(), glAPI.glSpecular(), 1.0, 1.0, 1.0, 1.0 );
     glAPI.Lightf(glAPI.glLight0(), glAPI.glPosition(), -200.0, 200.0, 100.0, 1.0);
     glAPI.Light(glAPI.glLight0(), glAPI.glSpotCutoff(),60.0);
     glAPI.Light(glAPI.glLight0(), glAPI.glSpotExponent(),50.0);
     glAPI.Lightf(glAPI.glLight0(), glAPI.glSpotDirection(), 0.3, -1.0, -0.3, 1.0);

     // Fairly shiny spot 2
     glAPI.LightModelv(glAPI.glLightModelAmbient(), 0.5, 0.5, 0.5, 0.5);
     glAPI.Lightf(glAPI.glLight1(), glAPI.glDiffuse(), 0.6, 0.6, 0.6, 0.6);
     glAPI.Lightf(glAPI.glLight1(), glAPI.glSpecular(), 1.0, 1.0, 1.0, 1.0 );
     glAPI.Lightf(glAPI.glLight1(), glAPI.glPosition(), -75.0, 200.0, 150.0, 1.0);
     glAPI.Light(glAPI.glLight1(), glAPI.glSpotCutoff(),60.0);
     glAPI.Light(glAPI.glLight1(), glAPI.glSpotExponent(),50.0);
     glAPI.Lightf(glAPI.glLight1(), glAPI.glSpotDirection(), 0.3, -1.0, -0.5, 1.0);

     // Fairly shiny spot 3
     glAPI.LightModelv(glAPI.glLightModelAmbient(), 0.5, 0.5, 0.5, 0.5);
     glAPI.Lightf(glAPI.glLight2(), glAPI.glDiffuse(), 0.6, 0.6, 0.6, 0.6);
     glAPI.Lightf(glAPI.glLight2(), glAPI.glSpecular(), 1.0, 1.0, 1.0, 1.0 );
     glAPI.Lightf(glAPI.glLight2(), glAPI.glPosition(), 50.0, 200.0, 100.0, 1.0);
     glAPI.Light(glAPI.glLight2(), glAPI.glSpotCutoff(),60.0);
     glAPI.Light(glAPI.glLight2(), glAPI.glSpotExponent(),50.0);
     glAPI.Lightf(glAPI.glLight2(), glAPI.glSpotDirection(), 0.3, -1.0, -0.3, 1.0);

     // Enable the lights
     glAPI.Enable(glAPI.glLight0());
     glAPI.Enable(glAPI.glLight1());
     glAPI.Enable(glAPI.glLight2());


     glAPI.Enable(glAPI.glNormalize());
     glAPI.Enable(glAPI.glAutoNormal());

     glAPI.ColorMaterial(glAPI.glFront(), glAPI.glAmbientAndDiffuse());
     glAPI.Materialv(glAPI.glFront(), glAPI.glSpecular(),1.0, 1.0, 1.0, 1.0);
     glAPI.Material(glAPI.glFront(),glAPI.glShininess(), 10.0);
     glAPI.Flush();

     // Build all the display lists that will be needed
     BuildLists();

end;


procedure TForm1.BuildLists();
var
   listFirst: integer;
   poleObj: integer;
   x, z, floorsize, division, step: real;
begin
     floorsize := 200.0;
     division := 10.0;
     step := floorsize/division;


   // Get a Range of display list names
   listFirst := glAPI.GenLists(10);

   // Assign and create the lists
   floorList := listFirst;
   poleList :=  listFirst + 1;
   ringListBase := listFirst + 2;  // There are eight of these

   // The Poles
   glAPI.NewList(poleList, glAPI.glCompile());
      glAPI.Color(255,0,0,255);
      poleObj := glAPI.gluNewQuadric();

      glAPI.gluQuadricNormals(poleObj,glAPI.gluFlat());
      glAPI.gluQuadricOrientation(poleObj,glAPI.gluOutside());
      glAPI.gluQuadricDrawStyle(poleObj,glAPI.gluFill());
      glAPI.gluCylinder(poleObj,7, 7, 60, 25, 25 ); // base, top, height, s/s

      glAPI.gluDeleteQuadric(poleObj);
   glAPI.EndList();

   // The floor
   glAPI.NewList(floorList, glAPI.glCompile());

     // White floor
     glAPI.Color(255, 255, 255, 255);

     // Tesselate floor or spot effect won't work
     x := -floorsize;
     z := -floorsize;

     while x < floorsize do begin
              while z < floorsize do begin
                       glAPI.Start(glAPI.glQuads());
                       glAPI.Normal(0.0, 1.0, 0.0);
                       glAPI.Vertex3(x, 0.0, z);
                       glAPI.Vertex3(x, 0.0, z+division);
                       glAPI.Vertex3(x+division, 0.0, z+division);
                       glAPI.Vertex3(x+division, 0.0, z);
                       glAPI.Stop();
                       z:= z + division;
                  end;
         x:= x + division;
         z:= -floorsize;
         end;
   glAPI.EndList();



   // Build the Rings
   // Large base ring (Red)
   glAPI.NewList(ringListBase,glAPI.glCompile());
     glAPI.Color(255, 0, 0, 255);
     glAPI.auxSolidTorus(10,25);
   glAPI.EndList();

   // Second biggest (Blue)
   glAPI.NewList(ringListBase+1,glAPI.glCompile());
     glAPI.Color(0, 0, 255, 255);
     glAPI.auxSolidTorus(9,22);
   glAPI.EndList();

   // Third biggest (green)
   glAPI.NewList(ringListBase+2,glAPI.glCompile());
     glAPI.Color(0, 255, 0, 255);
     glAPI.auxSolidTorus(8,19);
   glAPI.EndList();

   // 4rth (Yellow)
   glAPI.NewList(ringListBase+3,glAPI.glCompile());
     glAPI.Color(255, 255, 0, 255);
     glAPI.auxSolidTorus(7,16);
   glAPI.EndList();

   // 5th (Purple)
   glAPI.NewList(ringListBase+4,glAPI.glCompile());
     glAPI.Color(255, 0, 255, 255);
     glAPI.auxSolidTorus(6,13);
   glAPI.EndList();

   // 6th (Light Blue)
   glAPI.NewList(ringListBase+5,glAPI.glCompile());
     glAPI.Color(0, 255, 255, 255);
     glAPI.auxSolidTorus(5,9);
   glAPI.EndList();

   glAPI.Flush();

end;




procedure TForm1.glAPIRender(Sender: TObject);

begin
     glAPI.Clear(glAPI.glColorBufferBit());
     glAPI.Clear(glAPI.glDepthBufferBit());

     glAPI.Color(255, 0, 0, 255);

     glAPI.PushMatrix();
     glAPI.Rotate(90.0, 1.0, 0.0, 0.0);
     glAPI.CallList(poleList);

     glAPI.Translate(0,0,40);
     glAPI.CallList(ringListBase);

     glAPI.Translate(0, 0, -18);
     glAPI.CallList(ringListBase + 1);

     glAPI.Translate(0, 0, -17);
     glAPI.CallList(ringListBase + 2);

     glAPI.Translate(0, 0, -15);
     glAPI.CallList(ringListBase + 3);

     glAPI.Translate(0, 0, -13);
     glAPI.CallList(ringListBase + 4);

     glAPI.Translate(0, 0, -11);
     glAPI.CallList(ringListBase + 5);

     glAPI.PopMatrix();



     // Draw the floor
     glAPI.PushMatrix();
     glAPI.Translate(0.0, -75.0, 0.0);

     glAPI.CallList(floorList);

     glAPI.PopMatrix();

     glAPI.Flush();
     glAPI.SwapBuffers();
end;

procedure TForm1.FormActivate(Sender: TObject);
begin
     glAPI.MakeCurrent();
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
glAPI.MakeNotCurrent();
end;

end.
