// bounce2.c
// Animation with double buffering

#include <windows.h>	// Standard windows include
#include <gl\gl.h>		// OpenGL library
#include <gl\glaux.h>	// AUX library


// Initial square position and size
GLfloat x1 = 100.0f;
GLfloat y1 = 150.0f;
GLsizei rsize = 50;

// Step size in x and y directions
// (number of pixels to move each time)
GLfloat xstep = 1.0f;
GLfloat ystep = 1.0f;

// Keep track of windows changing width and height
GLfloat windowWidth;
GLfloat windowHeight;


// Called by AUX library when the window has changed size
void CALLBACK ChangeSize(GLsizei w, GLsizei h)
	{
	// Prevent a divide by zero, when window is too short
	// (you cant make a window of zero width).
	if(h == 0)
		h = 1;

	// Set the viewport to be the entire window
    glViewport(0, 0, w, h);

	// Reset the coordinate system before modifying
    glLoadIdentity();


	// Keep the square square, this time, save calculated
	// width and height for later use
	if (w <= h) 
		{
		windowHeight = 250.0f*h/w;
		windowWidth = 250.0f;
		}
    else 
		{
		windowWidth = 250.0f*w/h;
		windowHeight = 250.0f;
		}

	// Set the clipping volume
	glOrtho(0.0f, windowWidth, 0.0f, windowHeight, 1.0f, -1.0f);
	}

// Called by AUX library to update window
void CALLBACK RenderScene(void)
	{
	// Set background clearing color to blue
	glClearColor(0.0f, 0.0f, 1.0f, 1.0f);
	
	// Clear the window with current clearing color
	glClear(GL_COLOR_BUFFER_BIT);

	// Set drawing color to Red, and draw rectangle at
	// current position.
	glColor3f(1.0f, 0.0f, 0.0f);
	glRectf(x1, y1, x1+rsize, y1+rsize);

	glFlush();

	// Quickly blast image to window
	auxSwapBuffers();
	}


// Called by AUX library when idle (window not being
// resized or moved)
void CALLBACK IdleFunction(void)
	{
	// Reverse direction when you reach left or right edge
	if(x1 > windowWidth-rsize || x1 < 0)
		xstep = -xstep;

	// Reverse direction when you reach top or bottom edge
	if(y1 > windowHeight-rsize || y1 < 0)
		ystep = -ystep;


	// Check bounds.  This is incase the window is made
	// smaller and the rectangle is outside the new
	// clipping volume
	if(x1 > windowWidth-rsize)
		x1 = windowWidth-rsize-1;

	if(y1 > windowHeight-rsize)
		y1 = windowHeight-rsize-1;

	// Actually move the square
	x1 += xstep;
	y1 += ystep;

	// Redraw the scene with new coordinates
	RenderScene();
	}


// Main body of program
void main(void)
	{
	// AUX window setup and initialization,
	// now with double buffering
	auxInitDisplayMode(AUX_DOUBLE | AUX_RGBA);
	auxInitPosition(100,100,250,250);
	auxInitWindow("Double Buffered Animation");

	// Set function to call when window is resized
	auxReshapeFunc(ChangeSize);

	// Set function to call when program is idle
	auxIdleFunc(IdleFunction);
	
	// Start main loop
	auxMainLoop(RenderScene);
	}

