// KeyMove.c
// Move Rectangle responding to Keyboard input

#include <windows.h>	// Standard Windows header
#include <gl\gl.h>		// OpenGL Library
#include <gl\glaux.h>	// AUX Library


// Initial square position and size
GLfloat x1 = 100.0f;
GLfloat y1 = 150.0f;
GLfloat rsize = 50.0f;

GLfloat newWidth;
GLfloat newHeight;

// Called by AUX Library when the window has changed size
void CALLBACK ChangeSize(GLsizei w, GLsizei h)
	{
	// Prevent a divide by zero
	if(h == 0)
		h = 1;
		
	// Set Viewport to window dimensions
    glViewport(0, 0, w, h);

	// Reset coordinate system
	glLoadIdentity();

	// Keep the square square, this time, save calculated
	// width and height for later use
	if (w <= h) 
		{
		newHeight = 250.0f*h/w;
		newWidth = 250.0f;
		}
    else 
		{
		newWidth = 250.0f*w/h;
		newHeight = 250.0f;
		}

	// Set the clipping volume
	glOrtho(0.0f, newWidth, 0.0f, newHeight, 1.0f, -1.0f);
	}


// Called by the AUX Library whenever the window 
// needs to be updated
void CALLBACK RenderScene(void)
	{
	// Set background color to blue
	glClearColor(0.0f, 0.0f, 1.0f, 1.0f);

	// OpenGL or AUX Library drawing code
	glClear(GL_COLOR_BUFFER_BIT);

	// Draws a Red Rectangle at the current position
	glColor3f(1.0f, 0.0f, 0.0f);
	glRectf(x1, y1, x1+rsize, y1-rsize);

	glFlush();

	// Swap drawing to screen
	auxSwapBuffers();
	}


// Called by AUX library when up arrow is pressed
void CALLBACK KeyUp(void)
	{
	y1++;

	// Check to outside window
	if(y1 > newHeight)
		y1 = newHeight;
	}


// Called by AUX library when down arrow is pressed
void CALLBACK KeyDown(void)
	{
	y1--;

	// Check to outside window
	if(y1 - rsize < 0.0f)
		y1 = rsize;
	}


// Called by AUX library when left arrow is pressed
void CALLBACK KeyLeft(void)
	{
	x1--;

	// Check to outside window
	if(x1 < 0.0f)
		x1 = 0.0f;
	}


// Called by AUX library when right arrow is pressed
void CALLBACK KeyRight(void)
	{	
	x1++;

	// Check to outside window
	if(x1+rsize > newWidth)
		x1 = newWidth - rsize;
	}


// Main program body	
void main(void)
	{
	// Setup and initialize AUX window
	auxInitDisplayMode(AUX_DOUBLE | AUX_RGBA);
	auxInitPosition(100,100,250,250);
	auxInitWindow("Move with Arrow Keys");

	// Set function to call when window changes size
	auxReshapeFunc(ChangeSize);

	// Set functions for specific keystrokes
	auxKeyFunc(AUX_LEFT,KeyLeft);		// left arrow
	auxKeyFunc(AUX_RIGHT,KeyRight);		// right arrow
	auxKeyFunc(AUX_UP,KeyUp);			// up arrow
	auxKeyFunc(AUX_DOWN,KeyDown);		// down arror

	// Set function to call when window needs updating
	auxMainLoop(RenderScene);
	}

