// WinHint.c
// Demonstrates OpenGL Hints
// 

#include <windows.h>            // Window defines
#include <math.h>				// Include for sqrt()
#include <stdlib.h>				// for rand()
#include <gl\gl.h>              // OpenGL
#include <gl\glu.h>             // GLU library
#include <gl\glaux.h>
#include "resource.h"           // About box resource identifiers.

#include "bitmap.h"
#include "Texture.h"

// Lighting components /////////////////////////////
GLfloat  ambientLight[] = { 0.4f, 0.4f, 0.4f, 1.0f };
GLfloat  diffuseLight[] = { 0.6f, 0.6f, 0.6f, 1.0f };
GLfloat  specular[] = { 1.0f, 1.0f, 1.0f, 1.0f};
GLfloat	 lightPos[] = { -75.0f, 200.0f, 100.0f, 0.0f };



// Declaration for Window procedure
LRESULT CALLBACK WndProc(   HWND    hWnd,
							UINT    message,
							WPARAM  wParam,
							LPARAM  lParam);

// Dialog procedure for about box
BOOL APIENTRY AboutDlgProc (HWND hDlg, UINT message, UINT wParam, LONG lParam);

// Dialog procedure for control panel
BOOL APIENTRY ControlDlgProc (HWND hDlg, UINT message, UINT wParam, LONG lParam);

// Set Pixel Format function - forward declaration
void SetDCPixelFormat(HDC hDC);


#define glRGB(x, y, z)	glColor3ub((GLubyte)x, (GLubyte)y, (GLubyte)z)


HPALETTE hPalette = NULL;
HWND hMainWnd = NULL;

// Application name and instance storeage
static LPCTSTR lpszAppName = "OpenGL Hints";
static HINSTANCE hInstance;

// Generate a random number between nLower and nUpper (inclusive)
int RandInt(int nLower, int nUpper)
	{
	double dWorking;
	int nRange;
	
	if(nLower > 1)
		nRange = nUpper + nLower;
	else
		nRange = nUpper - nLower;

	// Get the Random number
	dWorking = rand();

	// Scale to between 0 and 1.0
	dWorking /= (double)RAND_MAX;
	if(dWorking > 1.0 || dWorking < 0.0)
		{
		dWorking = 1.0;
		return 0;
		}

	// Scale to range
	dWorking *= (double)nRange;
	if(nLower > 1)
		dWorking -= nLower;
	else
		dWorking += nLower;

	if(dWorking < nLower || dWorking > nUpper)
		return 0;

	return (int)dWorking;
	}



void ProcessHints(int nButton)
	{
	switch(nButton)
		{
		case IDC_FOG_NICE:
			glHint(GL_FOG_HINT, GL_NICEST);
			break;

		case IDC_FOG_FAST:
			glHint(GL_FOG_HINT, GL_FASTEST);
			break;

		case IDC_FOG_DC:
			glHint(GL_FOG_HINT, GL_DONT_CARE);
			break;

		case IDC_LINE_NICE:
			glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);
			break;

		case IDC_LINE_FAST:
			glHint(GL_LINE_SMOOTH_HINT, GL_FASTEST);
			break;

		case IDC_LINE_DC:
			glHint(GL_LINE_SMOOTH_HINT, GL_DONT_CARE);
			break;

		case IDC_PERS_NICE:
			glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
			break;

		case IDC_PERS_FAST:
			glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_FASTEST);
			break;

		case IDC_PERS_DC:
			glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_DONT_CARE);
			break;

		case IDC_POINT_NICE:
			glHint(GL_POINT_SMOOTH_HINT, GL_NICEST);
			break;

		case IDC_POINT_FAST:
			glHint(GL_POINT_SMOOTH_HINT, GL_FASTEST);
			break;

		case IDC_POINT_DC:
			glHint(GL_POINT_SMOOTH_HINT, GL_DONT_CARE);
			break;

		case IDC_POLY_NICE:
			glHint(GL_POLYGON_SMOOTH_HINT, GL_NICEST);
			break;

		case IDC_POLY_FAST:
			glHint(GL_POLYGON_SMOOTH_HINT, GL_FASTEST);
			break;

		case IDC_POLY_DC:
			glHint(GL_POLYGON_SMOOTH_HINT, GL_DONT_CARE);
			break;

		default:
			glHint(GL_POINT_SMOOTH_HINT, GL_DONT_CARE);
			glHint(GL_LINE_SMOOTH_HINT, GL_DONT_CARE);
			glHint(GL_POLYGON_SMOOTH_HINT, GL_DONT_CARE);
			glHint(GL_FOG_HINT, GL_DONT_CARE);
			glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_DONT_CARE);
			break;
		}
	}

// Draw a tree out of a green cone and a brown cylinder
void DrawTree(void)
	{
	GLUquadricObj *pTree;

	pTree = gluNewQuadric();
	gluQuadricNormals(pTree,GLU_SMOOTH);
	gluQuadricOrientation(pTree,GLU_OUTSIDE);
	gluQuadricDrawStyle(pTree,GLU_FILL);

	glPushMatrix();
	glRotatef(-90.0f, 1.0f, 0.0f, 0.0f);

	// Trunk
	glColor3f(0.6f, 0.4f, 0.12f);
	gluCylinder(pTree, 10.0f, 10.0f, 30.0f, 10, 10);
		
	// Foileage
	glRGB(0, 255, 0);
	glTranslatef(0.0f, 0.0f, 25.0f);
	gluCylinder(pTree, 30.0f, 0.0, 50.0f, 15, 15);
		
	gluDeleteQuadric(pTree);
	glPopMatrix();
	}


// The background stars are just randome sized points
void DrawStars(void)
	{
	int nCount;

	glDisable(GL_LIGHTING);
	glRGB(255, 255, 255);
	glBegin(GL_POINTS);
		for(nCount = 0; nCount < 15; nCount++)
			glVertex3i(RandInt(-450, 450), RandInt(-50, 250), -550);
	glEnd();

	glPointSize(3.0f);
	glBegin(GL_POINTS);
		for(nCount = 0; nCount < 12; nCount++)
			glVertex3i(RandInt(-450, 450), RandInt(-50, 250), -550);
	glEnd();

	glPointSize(5.0f);
	glBegin(GL_POINTS);
		for(nCount = 0; nCount < 10; nCount++)
			glVertex3i(RandInt(-450, 450), RandInt(-50, 250), -550);
	glEnd();


	glEnable(GL_LIGHTING);
	}



// Setup the rendering context and lighting conditions
void SetupRC(void)
	{
	glEnable(GL_DEPTH_TEST);	// Hidden surface removal
	glDepthFunc(GL_LEQUAL);

	glFrontFace(GL_CCW);		// Counter clock-wise polygons face out
	glCullFace(GL_BACK);
	glEnable(GL_CULL_FACE);
		
	// Black background
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f );

	// Setup Lighting
	glEnable(GL_LIGHTING);

	// Ambient light present
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambientLight);
	
	// Directional light also has a diffuse and specular component
	glLightfv(GL_LIGHT0,GL_DIFFUSE,diffuseLight);
	glLightfv(GL_LIGHT0,GL_SPECULAR,specular);
	glLightfv(GL_LIGHT0,GL_POSITION,lightPos);

	// Turn on the specific light
	glEnable(GL_LIGHT0);

	
	// Material ambient and diffuse reflectance tracks glColor
	glEnable(GL_COLOR_MATERIAL);
	glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
	glMaterialfv(GL_FRONT, GL_SPECULAR,specular);
	glMateriali(GL_FRONT,GL_SHININESS,10);

	// Let's let OpenGL worry about normals
	glEnable(GL_NORMALIZE);
	glEnable(GL_AUTO_NORMAL);

	// Blending, Antialiasing, Fog
	glEnable(GL_LINE_SMOOTH);
	glEnable(GL_POINT_SMOOTH);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	
	glEnable(GL_FOG);
	{
		GLfloat fogColor[4] = { 0.5f, 0.5f, 0.5f, 0.1f};
	
		glFogi(GL_FOG_MODE, GL_EXP2);
		glFogfv(GL_FOG_COLOR, fogColor);
		glFogf(GL_FOG_DENSITY, 0.0012f);
	
		glClearColor(0.f, 0.f, 0.f, 1.0f);
		}

	ProcessHints(0);

	glFlush();
	}





// Change viewing volume and viewport.  Called when window is resized
void ChangeSize(GLsizei w, GLsizei h)
	{
	GLfloat fAspect;
	
	// Adjust for dialog box
	h = h - 80;

	// Prevent a divide by zero
	if(h == 0)
		h = 1;

	fAspect = (float)w/(float)h;
		
	// Set Viewport to window dimensions
    glViewport(0, 80, w, h);

	// Reset coordinate system
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

    // Setup perspective for viewing
    gluPerspective(45.0, fAspect, 1, 1200);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

    // Viewing transformation, backup 100 units
    glTranslatef(0.0f, 0.0f, -200.0f);
	}



// The floor consists of a series of intersecting green lines
void DrawFloor(void)
	{
	GLfloat fFloorSize = 800.0f;
	GLfloat fDivision = 40.0f;
	GLfloat fX, fZ;

	glDisable(GL_LIGHTING);
	glRGB(0, 0, 255);

	for(fX = -fFloorSize; fX < fFloorSize; fX += fDivision)
		{
		glBegin(GL_LINES);
			glVertex3f(fX, 0.0f, fFloorSize);
			glVertex3f(fX, 0.0f, -fFloorSize);
		glEnd();
		}

	for(fZ = -fFloorSize; fZ < fFloorSize; fZ += fDivision)
		{
		glBegin(GL_LINES);
			glVertex3f(-fFloorSize, 0.0f, fZ);
			glVertex3f(fFloorSize, 0.0f, fZ);
		glEnd();
		}

	glEnable(GL_LIGHTING);


	glRGB(255,0,0);

	glEnable(GL_TEXTURE_2D);
	glPushMatrix();


	glTranslatef(30.0f, 0.0f, -225.0f);

	TextureLoadBitmap("checks.bmp");
	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);

	glBegin(GL_QUADS);
		glNormal3f(0.0f, 1.0f, 0.0f);

		glTexCoord2f(0.0f, 1.0f);
		glVertex3f(0.0f, 10.0f, 0.0f);

		glTexCoord2f(1.0f, 1.0f);
		glVertex3f(300.0f, 10.0f, 0.0f);

		glTexCoord2f(1.0f, 0.0f);
		glVertex3f(300.0f, 2.0f, -300.0f);

		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(0.0f, .0f, -300.0f);

	glEnd();

	glPopMatrix();
	glDisable(GL_TEXTURE_2D);


	}


// Called to draw scene
void RenderScene(void)
	{
	// Clear the window with current clearing color
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	// Reposition the light after any viewing transforms
	glLightfv(GL_LIGHT0,GL_POSITION,lightPos);

	// Draw the background stars
	DrawStars();

	glPushMatrix();

	// Draw the floor
	glTranslatef(0.0f, -100.0f, 0.0f);
	DrawFloor();

	glPushMatrix();
	glTranslatef(-75.0f, 0.0f, 25.0f);
	DrawTree();
	glPopMatrix();

	// Place the trees. These translations are cumulative
	glTranslatef(0.0f, 0.0f, -200.0f);
	DrawTree();

	glTranslatef(200.0f, 0.0f, 0.0f);
	DrawTree();

	glTranslatef(-400.0f, 0.0f, -200.0f);
	DrawTree();

	glTranslatef(400.0f, 0.0f, -200.0f);
	DrawTree();

	glTranslatef(-700.0f, 0.0f, -150.0f);
	DrawTree();

	glTranslatef(1100.0f, 0.0f, 0.0f);
	DrawTree();

	glPopMatrix();

	// Polygon antialiasing
	glPushMatrix();
	
	glFlush();


	// Antialiased Cubs
	glEnable(GL_POLYGON_SMOOTH);
	glDisable(GL_DEPTH_TEST);

	glColor4f(1.0f, 0.0f, 0.0f, 1.0f);
	glBlendFunc(GL_SRC_ALPHA_SATURATE, GL_ONE);
		
	glRotatef(30.0f, 1.0f, 0.0f, 0.0f);
	glRotatef(60.0f, 0.0f, 1.0f, 0.0f);
	
	glTranslatef(50.0f, -100.0f, -75.0f);
	auxSolidCube(25.0f);

	glTranslatef(200.0f, 100.0f, -200.0f);
	auxSolidCube(25.0f);

	glTranslatef(-150.0f, -100.0f, 0.0f);
	auxSolidCube(25.0f);

	glDisable(GL_POLYGON_SMOOTH);
	glEnable(GL_DEPTH_TEST);

	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glPopMatrix();
	
	// Flush drawing commands
	glFlush();
	}


// Select the pixel format for a given device context
void SetDCPixelFormat(HDC hDC)
	{
	int nPixelFormat;

	static PIXELFORMATDESCRIPTOR pfd = {
		sizeof(PIXELFORMATDESCRIPTOR),  // Size of this structure
		1,                                                              // Version of this structure    
		PFD_DRAW_TO_WINDOW |                    // Draw to Window (not to bitmap)
		PFD_SUPPORT_OPENGL |					// Support OpenGL calls in window
		PFD_DOUBLEBUFFER,                       // Double buffered
		PFD_TYPE_RGBA,                          // RGBA Color mode
		24,                                     // Want 24bit color 
		0,0,0,0,0,0,                            // Not used to select mode
		0,0,                                    // Not used to select mode
		0,0,0,0,0,                              // Not used to select mode
		32,                                     // Size of depth buffer
		0,                                      // Not used to select mode
		0,                                      // Not used to select mode
		PFD_MAIN_PLANE,                         // Draw in main plane
		0,                                      // Not used to select mode
		0,0,0 };                                // Not used to select mode

	// Choose a pixel format that best matches that described in pfd
	nPixelFormat = ChoosePixelFormat(hDC, &pfd);

	// Set the pixel format for the device context
	SetPixelFormat(hDC, nPixelFormat, &pfd);
	}



// If necessary, creates a 3-3-2 palette for the device context listed.
HPALETTE GetOpenGLPalette(HDC hDC)
	{
	HPALETTE hRetPal = NULL;	// Handle to palette to be created
	PIXELFORMATDESCRIPTOR pfd;	// Pixel Format Descriptor
	LOGPALETTE *pPal;			// Pointer to memory for logical palette
	int nPixelFormat;			// Pixel format index
	int nColors;				// Number of entries in palette
	int i;						// Counting variable
	BYTE RedRange,GreenRange,BlueRange;
								// Range for each color entry (7,7,and 3)


	// Get the pixel format index and retrieve the pixel format description
	nPixelFormat = GetPixelFormat(hDC);
	DescribePixelFormat(hDC, nPixelFormat, sizeof(PIXELFORMATDESCRIPTOR), &pfd);

	// Does this pixel format require a palette?  If not, do not create a
	// palette and just return NULL
	if(!(pfd.dwFlags & PFD_NEED_PALETTE))
		return NULL;

	// Number of entries in palette.  8 bits yeilds 256 entries
	nColors = 1 << pfd.cColorBits;	

	// Allocate space for a logical palette structure plus all the palette entries
	pPal = (LOGPALETTE*)malloc(sizeof(LOGPALETTE) +nColors*sizeof(PALETTEENTRY));

	// Fill in palette header 
	pPal->palVersion = 0x300;		// Windows 3.0
	pPal->palNumEntries = nColors; // table size

	// Build mask of all 1's.  This creates a number represented by having
	// the low order x bits set, where x = pfd.cRedBits, pfd.cGreenBits, and
	// pfd.cBlueBits.  
	RedRange = (1 << pfd.cRedBits) -1;
	GreenRange = (1 << pfd.cGreenBits) - 1;
	BlueRange = (1 << pfd.cBlueBits) -1;

	// Loop through all the palette entries
	for(i = 0; i < nColors; i++)
		{
		// Fill in the 8-bit equivalents for each component
		pPal->palPalEntry[i].peRed = (i >> pfd.cRedShift) & RedRange;
		pPal->palPalEntry[i].peRed = (unsigned char)(
			(double) pPal->palPalEntry[i].peRed * 255.0 / RedRange);

		pPal->palPalEntry[i].peGreen = (i >> pfd.cGreenShift) & GreenRange;
		pPal->palPalEntry[i].peGreen = (unsigned char)(
			(double)pPal->palPalEntry[i].peGreen * 255.0 / GreenRange);

		pPal->palPalEntry[i].peBlue = (i >> pfd.cBlueShift) & BlueRange;
		pPal->palPalEntry[i].peBlue = (unsigned char)(
			(double)pPal->palPalEntry[i].peBlue * 255.0 / BlueRange);

		pPal->palPalEntry[i].peFlags = (unsigned char) NULL;
		}
		

	// Create the palette
	hRetPal = CreatePalette(pPal);

	// Go ahead and select and realize the palette for this device context
	SelectPalette(hDC,hRetPal,FALSE);
	RealizePalette(hDC);

	// Free the memory used for the logical palette structure
	free(pPal);

	// Return the handle to the new palette
	return hRetPal;
	}


// Entry point of all Windows programs
int APIENTRY WinMain(   HINSTANCE       hInst,
						HINSTANCE       hPrevInstance,
						LPSTR           lpCmdLine,
						int                     nCmdShow)
	{
	MSG                     msg;            // Windows message structure
	WNDCLASS        wc;                     // Windows class structure
	HWND            hWnd;           // Storeage for window handle

	hInstance = hInst;

	// Register Window style
	wc.style                = CS_HREDRAW | CS_VREDRAW;
	wc.lpfnWndProc          = (WNDPROC) WndProc;
	wc.cbClsExtra           = 0;
	wc.cbWndExtra           = 0;
	wc.hInstance            = hInstance;
	wc.hIcon                = NULL;
	wc.hCursor              = LoadCursor(NULL, IDC_ARROW);
	
	// No need for background brush for OpenGL window
	wc.hbrBackground        = NULL;         
	
	wc.lpszMenuName         = MAKEINTRESOURCE(IDR_MENU);
	wc.lpszClassName        = lpszAppName;

	// Register the window class
	if(RegisterClass(&wc) == 0)
		return FALSE;


	// Create the main application window
	hWnd = CreateWindow(
				lpszAppName,
				lpszAppName,
				
				// OpenGL requires WS_CLIPCHILDREN and WS_CLIPSIBLINGS
				WS_CAPTION | WS_BORDER | WS_SYSMENU | WS_CLIPCHILDREN | WS_CLIPSIBLINGS,
	
				// Window position and size
				0, 0,
				581, 449,
				NULL,
				NULL,
				hInstance,
				NULL);

	// If window was not created, quit
	if(hWnd == NULL)
		return FALSE;

	hMainWnd = hWnd;

	// Display the window
	ShowWindow(hWnd,SW_SHOW);
	UpdateWindow(hWnd);

	// Process application messages until the application closes
	while( GetMessage(&msg, NULL, 0, 0))
		{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
		}

	return msg.wParam;
	}




// Window procedure, handles all messages for this program
LRESULT CALLBACK WndProc(       HWND    hWnd,
							UINT    message,
							WPARAM  wParam,
							LPARAM  lParam)
	{
	static HGLRC hRC;               // Permenant Rendering context
	static HDC hDC;                 // Private GDI Device context
	static HWND hControlsWnd = NULL;	// Window with controls

	switch (message)
		{
		// Window creation, setup for OpenGL
		case WM_CREATE:
			// Store the device context
			hDC = GetDC(hWnd);              

			// Select the pixel format
			SetDCPixelFormat(hDC);          

			hPalette = GetOpenGLPalette(hDC);

			// Create the rendering context and make it current
			hRC = wglCreateContext(hDC);
			wglMakeCurrent(hDC, hRC);
			SetupRC();


			// Create control panel
			hControlsWnd = CreateDialog (hInstance,
				MAKEINTRESOURCE(IDD_DLG_SETTINGS),
				hWnd,
				ControlDlgProc);

			break;

		// Window is being destroyed, cleanup
		case WM_DESTROY:
			// Deselect the current rendering context and delete it
			wglMakeCurrent(hDC,NULL);
			wglDeleteContext(hRC);

			if(hPalette != NULL)
				DeleteObject(hPalette);

			if(hControlsWnd)
				DestroyWindow(hControlsWnd);

			// Tell the application to terminate after the window
			// is gone.
			PostQuitMessage(0);
			break;

		// Window is resized.
		case WM_SIZE:
			// Call our function which modifies the clipping
			// volume and viewport
			ChangeSize(LOWORD(lParam), HIWORD(lParam));
			break;


		// The painting function.  This message sent by Windows 
		// whenever the screen needs updating.
		case WM_PAINT:
			{
			// Call OpenGL drawing code
			RenderScene();

			SwapBuffers(hDC);

			// Validate the newly painted client area
			ValidateRect(hWnd,NULL);
			}
			break;

		// Windows is telling the application that it may modify
		// the system palette.  This message in essance asks the 
		// application for a new palette.
		case WM_QUERYNEWPALETTE:
			// If the palette was created.
			if(hPalette)
				{
				int nRet;

				// Selects the palette into the current device context
				SelectPalette(hDC, hPalette, FALSE);

				// Map entries from the currently selected palette to
				// the system palette.  The return value is the number 
				// of palette entries modified.
				nRet = RealizePalette(hDC);

				// Repaint, forces remap of palette in current window
				InvalidateRect(hWnd,NULL,FALSE);

				return nRet;
				}
			break;

	
		// This window may set the palette, even though it is not the 
		// currently active window.
		case WM_PALETTECHANGED:
			// Don't do anything if the palette does not exist, or if
			// this is the window that changed the palette.
			if((hPalette != NULL) && ((HWND)wParam != hWnd))
				{
				// Select the palette into the device context
				SelectPalette(hDC,hPalette,FALSE);

				// Map entries to system palette
				RealizePalette(hDC);
				
				// Remap the current colors to the newly realized palette
				UpdateColors(hDC);
				return 0;
				}
			break;

		// A menu command
		case WM_COMMAND:
			{
			switch(LOWORD(wParam))
				{
				// Exit the program
				case ID_FILE_EXIT:
					DestroyWindow(hWnd);
					break;

				// Display the about box
				case ID_HELP_ABOUT:
					DialogBox (hInstance,
						MAKEINTRESOURCE(IDD_DIALOG_ABOUT),
						hWnd,
						AboutDlgProc);
					break;
				}
			}
			break;


	default:   // Passes it on if unproccessed
	    return (DefWindowProc(hWnd, message, wParam, lParam));

	}

    return (0L);
	}




// Dialog procedure for keywatch control panel dialog
BOOL APIENTRY AboutDlgProc (HWND hDlg, UINT message, UINT wParam, LONG lParam)
	{
	
    switch (message)
	{
		// Initialize the dialog box
	    case WM_INITDIALOG:
			{
			int i;
			GLenum glError;

			// glGetString demo
			SetDlgItemText(hDlg,IDC_OPENGL_VENDOR,glGetString(GL_VENDOR));
			SetDlgItemText(hDlg,IDC_OPENGL_RENDERER,glGetString(GL_RENDERER));
			SetDlgItemText(hDlg,IDC_OPENGL_VERSION,glGetString(GL_VERSION));
			SetDlgItemText(hDlg,IDC_OPENGL_EXTENSIONS,glGetString(GL_EXTENSIONS));

			// gluGetString demo
			SetDlgItemText(hDlg,IDC_GLU_VERSION,gluGetString(GLU_VERSION));
			SetDlgItemText(hDlg,IDC_GLU_EXTENSIONS,gluGetString(GLU_EXTENSIONS));


			// Display any recent error messages
			i = 0;
			do {
				glError = glGetError();
				SetDlgItemText(hDlg,IDC_ERROR1+i,gluErrorString(glError));
				i++;
				}
			while(i < 6 && glError != GL_NO_ERROR);


			return (TRUE);
			}
			break;

		// Process command messages
	    case WM_COMMAND:      
			{
			// Validate and Make the changes
			if(LOWORD(wParam) == IDOK)
				EndDialog(hDlg,TRUE);
		    }
			break;

		// Closed from sysbox
		case WM_CLOSE:
			EndDialog(hDlg,TRUE);
			break;
		}

	return FALSE;
	}




// Dialog procedure for Radio buttons panel
BOOL APIENTRY ControlDlgProc (HWND hDlg, UINT message, UINT wParam, LONG lParam)
	{
    switch (message)
		{
		// Initialize the dialog box
	    case WM_INITDIALOG:
			{
			// Set Radio Buttons to Don't Care
			//	IDC_FOG_DC
			//SendMessage(GetDlgItem(hDlg, IDC_FOG_DC),WM_CHECK);
			CheckDlgButton(hDlg, IDC_FOG_DC, 1);
			CheckDlgButton(hDlg, IDC_LINE_DC, 1);
			CheckDlgButton(hDlg, IDC_PERS_DC, 1);
			CheckDlgButton(hDlg, IDC_POINT_DC, 1);
			CheckDlgButton(hDlg, IDC_POLY_DC, 1);
	
			return (TRUE);
			}
			break;

		// Process command messages
	    case WM_COMMAND:      
			{
			int idButton;

			if(HIWORD(wParam) == BN_CLICKED)
				{
				// ID of the button clicked
				idButton = (int) LOWORD(wParam);
				ProcessHints(idButton);
				InvalidateRect(hMainWnd,NULL,FALSE);
				}
		    }
			break;
		}

	return FALSE;
	}
