// ColorDX.c
// Demonstrates color index mode and color cycling

#include <windows.h>	// Standard Windows header
#include <gl\gl.h>		// OpenGL Library
#include <gl\glaux.h>	// AUX Library


// Called by AUX Library when the window has changed size
void CALLBACK ChangeSize(GLsizei w, GLsizei h)
	{
	// Prevent a divide by zero
	if(h == 0)
		h = 1;
		
	// Set Viewport to window dimensions
    glViewport(0, 0, w, h);

	// Reset coordinate system
	glLoadIdentity();

	// Establish clipping volume (left, right, bottom, top, near, far)
    if (w <= h) 
		glOrtho (0.0f, 250.0f, 0.0f, 250.0f*h/w, 1.0, -1.0);
    else 
		glOrtho (0.0f, 250.0f*w/h, 0.0f, 250.0f, 1.0, -1.0);

	}


// Called by the AUX Library whenever the window 
// needs to be updated
void CALLBACK RenderScene(void)
	{
	int c;

	// Clear the screen
	glClear(GL_COLOR_BUFFER_BIT);

	// Draw a chain of squares each a different color index.
	for(c = 0; c < 7; c++)
		{
		glIndexi(c+1);
		glRectf(c*35.0f,100.0f,c*35.0f+35.0f,150.0f);
		}

	glFlush();
	}



// Called continuously by idle function
void CALLBACK ShiftPalette(void)
	{
	static float palette[][3] = { { 0.0f, 0.0f, 0.0f },
						   { 0.0f, 0.0f, 1.0f },
						   { 0.0f, 1.0f, 0.0f },
						   { 0.0f, 1.0f, 1.0f },
						   { 1.0f, 0.0f, 0.0f },
						   { 1.0f, 0.0f, 1.0f },
						   { 1.0f, 1.0f, 0.0f },
						   { 1.0f, 1.0f, 1.0f }};


	int i;
	float r,g,b;

	// Set palette
	for( i = 1; i < 9; i++)
		auxSetOneColor(i,palette[i-1][0],palette[i-1][1],palette[i-1][2]);
	
	// Shift palette for next time
	r = palette[0][0];
	g = palette[0][1];
	b = palette[0][2];

	for(i = 0; i < 7; i++)
		{
		palette[i][0] = palette[i+1][0];
		palette[i][1] = palette[i+1][1];
		palette[i][2] = palette[i+1][2];
		}

	palette[7][0] = r;
	palette[7][1] = g;
	palette[7][2] = b;

	// Some delay (250 milliseconds) to slow it down
	Sleep(250);
	}


void main(void)
	{
	// Setup and initialize AUX window
	auxInitDisplayMode(AUX_SINGLE | AUX_INDEX);
	auxInitPosition(100,100,250,250);
	auxInitWindow("Color Cycling");

	// Set function to call when window changes size
	auxReshapeFunc(ChangeSize);

	// Set Idle function which changes palette
	auxIdleFunc(ShiftPalette);

	// Set function to call when window needs updating
	auxMainLoop(RenderScene);
	}

