// Window procedure for Child Window #1

#include <windows.h>            // Window defines
#include <gl\gl.h>              // OpenGL
#include <gl\glu.h>             // GLU library


// Palette handle, created by main window
extern HPALETTE hPalette;
extern HWND hChild1;
extern HGLRC hRC1;               // Permenant Rendering context
// Declaration for Window procedure
LRESULT CALLBACK WndProc1(   HWND    hWnd,
							UINT    message,
							WPARAM  wParam,
							LPARAM  lParam);






void CreateChild1(HWND hParent, HINSTANCE hInst)
	{
	WNDCLASS        wc;                     // Windows class structure

	// Register Window style
	wc.style                = CS_HREDRAW | CS_VREDRAW  | CS_OWNDC;
	wc.lpfnWndProc          = (WNDPROC) WndProc1;
	wc.cbClsExtra           = 0;
	wc.cbWndExtra           = 0;
	wc.hInstance            = hInst;
	wc.hIcon                = NULL;
	wc.hCursor              = LoadCursor(NULL, IDC_ARROW);

	// No need for background brush for OpenGL window
	wc.hbrBackground        = NULL;

	wc.lpszMenuName         = NULL;
	wc.lpszClassName        = "Child1Class";

	// Register the window class
	RegisterClass(&wc);


	// Create the main application window
	hChild1 = CreateWindow(
				wc.lpszClassName,
				"Child1",

				// OpenGL requires WS_CLIPCHILDREN and WS_CLIPSIBLINGS
				WS_CHILD | WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN | WS_CLIPSIBLINGS,

				// Window position and size
				10, 10,
				250, 250,
				hParent,
				NULL,
				hInst,
				NULL);

  	// Display the window
	ShowWindow(hChild1,SW_SHOW);
	UpdateWindow(hChild1);
	  }





// Window procedure, handles all messages for this program
LRESULT CALLBACK WndProc1(       HWND    hWnd,
							UINT    message,
							WPARAM  wParam,
							LPARAM  lParam)
	{
	static HDC hDC;                 // Private GDI Device context

	switch (message)
		{
		// Window creation, setup for OpenGL
		case WM_CREATE:
			// Store the device context
			hDC = GetDC(hWnd);

			// Select the pixel format
			SetDCPixelFormat(hDC);

			// Create the rendering context and make it current
			hRC1 = wglCreateContext(hDC);
			wglMakeCurrent(hDC, hRC1);
			SetupRC();

         SetTimer(hWnd,101,1000,NULL);

			break;

      case WM_TIMER:
      	InvalidateRect(hWnd,NULL,FALSE);
      	break;

		// Window is being destroyed, cleanup
		case WM_DESTROY:
			// Deselect the current rendering context and delete it
			wglMakeCurrent(hDC,NULL);
			wglDeleteContext(hRC1);
			break;

		// Window is resized.
		case WM_SIZE:
			// Call our function which modifies the clipping
			// volume and viewport
 			wglMakeCurrent(hDC, hRC1);
			ChangeSize(LOWORD(lParam), HIWORD(lParam));
         wglMakeCurrent(hDC, NULL);
			break;


		// The painting function.  This message sent by Windows
		// whenever the screen needs updating.
		case WM_PAINT:
			{
         RECT rect;
         GetClientRect(hWnd,&rect);

  			wglMakeCurrent(hDC, hRC1);

			// Call OpenGL drawing code
      		// Rotate and translate the coordinate system
		// Note this will be cumulative
		glRotatef(5.0f, 0.0f, 1.0f, 0.0f);

		RenderScene();

		// Bring image to front
		SwapBuffers(hDC);

  		wglMakeCurrent(hDC, NULL);

			// Do not validate, forcing a continuous repaint
         ValidateRect(hWnd,&rect);
			}
			break;

		// Windows is telling the application that it may modify
		// the system palette.  This message in essance asks the
		// application for a new palette.
		case WM_QUERYNEWPALETTE:
			// If the palette was created.
			if(hPalette)
				{
				int nRet;

				// Selects the palette into the current device context
				SelectPalette(hDC, hPalette, FALSE);

				// Map entries from the currently selected palette to
				// the system palette.  The return value is the number
				// of palette entries modified.
				nRet = RealizePalette(hDC);

				// Validate window contents
				ValidateRect(hWnd,FALSE);

				return nRet;
				}
			break;

      case WM_ERASEBKGND:
      	return FALSE;
         break;

		// This window may set the palette, even though it is not the
		// currently active window.
		case WM_PALETTECHANGED:
			// Don't do anything if the palette does not exist, or if
			// this is the window that changed the palette.
			if((hPalette != NULL) && ((HWND)wParam != hWnd))
				{
				// Select the palette into the device context
				SelectPalette(hDC,hPalette,FALSE);

				// Map entries to system palette
				RealizePalette(hDC);

				// Remap the current colors to the newly realized palette
				UpdateColors(hDC);
				return 0;
				}
			break;

	default:   // Passes it on if unproccessed
	    return (DefWindowProc(hWnd, message, wParam, lParam));

	}

    return (0L);
	}
