// Window procedure for Child Window #2

#include <windows.h>            // Window defines
#include <gl\gl.h>              // OpenGL
#include <gl\glu.h>             // GLU library


#define glRGB(x, y, z)	glColor3ub((GLubyte)x, (GLubyte)y, (GLubyte)z)

// Palette handle, created by main window
extern HPALETTE hPalette;
extern HWND hChild2;
extern HGLRC hRC2;

// Declaration for Window procedure
LRESULT CALLBACK WndProc2(   HWND    hWnd,
							UINT    message,
							WPARAM  wParam,
							LPARAM  lParam);


// This function does any needed initialization on the rendering
// context.  Here it sets up and initializes the lighting for
// the scene.
void SetupRC2()
	{
	// Light values and coordinates
	GLfloat  ambientLight[] = { 0.3f, 0.3f, 0.3f, 1.0f };
	GLfloat  diffuseLight[] = { 0.7f, 0.7f, 0.7f, 1.0f };
	GLfloat  specular[] = { 1.0f, 1.0f, 1.0f, 1.0f};
	GLfloat	 lightPos[] = { 0.0f, 150.0f, 150.0f, 1.0f };
	GLfloat  specref[] =  { 1.0f, 1.0f, 1.0f, 1.0f };


	glEnable(GL_DEPTH_TEST);	// Hidden surface removal
	glFrontFace(GL_CCW);		// Counter clock-wise polygons face out
	glEnable(GL_CULL_FACE);		// Do not calculate inside of jet

	// Enable lighting
	glEnable(GL_LIGHTING);

	// Setup and enable light 0
	glLightfv(GL_LIGHT0,GL_AMBIENT,ambientLight);
	glLightfv(GL_LIGHT0,GL_DIFFUSE,diffuseLight);
	glLightfv(GL_LIGHT0,GL_SPECULAR,specular);
	glLightfv(GL_LIGHT0,GL_POSITION,lightPos);
	glEnable(GL_LIGHT0);

	// Enable color tracking
	glEnable(GL_COLOR_MATERIAL);
	
	// Set Material properties to follow glColor values
	glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);

	// All materials hereafter have full specular reflectivity
	// with a high shine
	glMaterialfv(GL_FRONT, GL_SPECULAR,specref);
	glMateriali(GL_FRONT,GL_SHININESS,128);


	// Light blue background
	glClearColor(0.0f, 0.0f, 1.0f, 1.0f );
	}


// Called to draw scene
void RenderScene2(void)
	{
	float normal[3];	// Storeage for calculated surface normal

	// Clear the window with current clearing color
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	// Save the matrix state and do the rotations
	glPushMatrix();

	// Nose Cone /////////////////////////////
	// Set material color
	glRGB(0, 255, 0);
	glBegin(GL_TRIANGLES);
		glNormal3f(0.0f, -1.0f, 0.0f);
		glVertex3f(0.0f, 0.0f, 60.0f);
		glVertex3f(-15.0f, 0.0f, 30.0f);
		glVertex3f(15.0f,0.0f,30.0f);
	//glEnd();

	
	{
	// Verticies for this panel
	float v[3][3] =	{{ 15.0f, 0.0f, 30.0f},
					{ 0.0f, 15.0f, 30.0f},
					{ 0.0f, 0.0f,	60.0f}};

	// Calculate the normal for the plane
	calcNormal(v,normal);

	// Draw the triangle using the plane normal
	// for all the vertices
	//glBegin(GL_TRIANGLES);
		glNormal3fv(normal);
		glVertex3fv(v[0]);
		glVertex3fv(v[1]);
		glVertex3fv(v[2]);
	//glEnd();
	}	


	{
	float v[3][3] = {{ 0.0f, 0.0f, 60.0f },
					 { 0.0f, 15.0f, 30.0f },
					 { -15.0f, 0.0f, 30.0f }};

	calcNormal(v,normal);

	//glBegin(GL_TRIANGLES);
		glNormal3fv(normal);
		glVertex3fv(v[0]);
		glVertex3fv(v[1]);
		glVertex3fv(v[2]);
	//glEnd();	
	}



	// Body of the Plane ////////////////////////
	// light gray
	glRGB(192,192,192);
	{
	float v[3][3] = {{ -15.0f,0.0f,30.0f },
				 { 0.0f, 15.0f, 30.0f },
				 { 0.0f, 0.0f, -56.0f }};

	calcNormal(v,normal);
	//glBegin(GL_TRIANGLES);
		glNormal3fv(normal);
		glVertex3fv(v[0]);
		glVertex3fv(v[1]);
		glVertex3fv(v[2]);
	//glEnd();
	}
	
	{
	float v[3][3] = {{ 0.0f, 0.0f, -56.0f },
					 { 0.0f, 15.0f, 30.0f },
					 { 15.0f,0.0f,30.0f }};
	calcNormal(v,normal);
	//glBegin(GL_TRIANGLES);
		glNormal3fv(normal);
		glVertex3fv(v[0]);
		glVertex3fv(v[1]);
		glVertex3fv(v[2]);	
	//glEnd();
	}
	
		
	//glBegin(GL_TRIANGLES);
		glNormal3f(0.0f, -1.0f, 0.0f);
		glVertex3f(15.0f,0.0f,30.0f);
		glVertex3f(-15.0f, 0.0f, 30.0f);
		glVertex3f(0.0f, 0.0f, -56.0f);
	//glEnd();


	///////////////////////////////////////////////
	// Left wing
	// Large triangle for bottom of wing
	// Dark gray

	// Set drawing color
	glRGB(128,128,128);
	{
	float v[3][3] = {{ 0.0f,2.0f,27.0f },
				 { -60.0f, 2.0f, -8.0f },
				 { 60.0f, 2.0f, -8.0f }};

	// Calculate the normal from the verticies
	calcNormal(v,normal);

	//glBegin(GL_TRIANGLES);
		glNormal3fv(normal);
		glVertex3fv(v[0]);
		glVertex3fv(v[1]);
		glVertex3fv(v[2]);
	//glEnd();
	}
	
	{
	float v[3][3] =    {{ 60.0f, 2.0f, -8.0f},
					{0.0f, 7.0f, -8.0f},
					{0.0f,2.0f,27.0f }};

	calcNormal(v,normal);

//	glBegin(GL_TRIANGLES);
		glNormal3fv(normal);
		glVertex3fv(v[0]);
		glVertex3fv(v[1]);
		glVertex3fv(v[2]);
//	glEnd();
	}

	{
	float v[3][3] = {{60.0f, 2.0f, -8.0f},
					{-60.0f, 2.0f, -8.0f},
					{0.0f,7.0f,-8.0f }};

	calcNormal(v,normal);

//	glBegin(GL_TRIANGLES);
		glNormal3fv(normal);
		glVertex3fv(v[0]);
		glVertex3fv(v[1]);
		glVertex3fv(v[2]);
//	glEnd();
	}

	{
	float v[3][3] = {{0.0f,2.0f,27.0f},
					 {0.0f, 7.0f, -8.0f},
					 {-60.0f, 2.0f, -8.0f}};

	calcNormal(v,normal);
	
	// Other wing top section
//	glBegin(GL_TRIANGLES);
		glNormal3fv(normal);
		glVertex3fv(v[0]);
		glVertex3fv(v[1]);
		glVertex3fv(v[2]);
//	glEnd();
	}


	// Tail section///////////////////////////////
	// Bottom of back fin
	glRGB(255,255,0);
	//glBegin(GL_TRIANGLES);
		glNormal3f(0.0f, -1.0f, 0.0f);
		glVertex3f(-30.0f, -0.50f, -57.0f);
		glVertex3f(30.0f, -0.50f, -57.0f);
		glVertex3f(0.0f,-0.50f,-40.0f);
	//glEnd();

	{
	float v[3][3] = {{ 0.0f,-0.5f,-40.0f },
					{30.0f, -0.5f, -57.0f},
					{0.0f, 4.0f, -57.0f }};

	calcNormal(v,normal);

	// top of left side
//	glBegin(GL_TRIANGLES);
		glNormal3fv(normal);
		glVertex3fv(v[0]);
		glVertex3fv(v[1]);
		glVertex3fv(v[2]);
//	glEnd();
	}

	{
	float v[3][3] = {{ 0.0f, 4.0f, -57.0f },
					{ -30.0f, -0.5f, -57.0f },
					{ 0.0f,-0.5f,-40.0f }};

	calcNormal(v,normal);

	// top of right side
//	glBegin(GL_TRIANGLES);
		glNormal3fv(normal);
		glVertex3fv(v[0]);
		glVertex3fv(v[1]);
		glVertex3fv(v[2]);
//	glEnd();
	}

	{
	float v[3][3] = {{ 30.0f,-0.5f,-57.0f },
					{ -30.0f, -0.5f, -57.0f },
					{ 0.0f, 4.0f, -57.0f }};

	calcNormal(v,normal);

	// back of bottom of tail
//	glBegin(GL_TRIANGLES);
		glNormal3fv(normal);
		glVertex3fv(v[0]);
		glVertex3fv(v[1]);
		glVertex3fv(v[2]);
//	glEnd();
	}

	{
	float v[3][3] = {{ 0.0f,0.5f,-40.0f },
					{ 3.0f, 0.5f, -57.0f },
					{ 0.0f, 25.0f, -65.0f }};

	calcNormal(v,normal);

	// Top of Tail section left
	glRGB(255,0,0);
//	glBegin(GL_TRIANGLES);
		glNormal3fv(normal);
		glVertex3fv(v[0]);
		glVertex3fv(v[1]);
		glVertex3fv(v[2]);
//	glEnd();
	}

	{
	float v[3][3] = {{ 0.0f, 25.0f, -65.0f },
					{ -3.0f, 0.5f, -57.0f},
					{ 0.0f,0.5f,-40.0f }};

	calcNormal(v,normal);

//	glBegin(GL_TRIANGLES);
		glNormal3fv(normal);
		glVertex3fv(v[0]);
		glVertex3fv(v[1]);
		glVertex3fv(v[2]);
//	glEnd();
	}

	{
	float v[3][3] = {{ 3.0f,0.5f,-57.0f },
					{ -3.0f, 0.5f, -57.0f },
					{ 0.0f, 25.0f, -65.0f }};

	calcNormal(v,normal);

	// Back of horizontal section
//	glBegin(GL_TRIANGLES);
		glNormal3fv(normal);
		glVertex3fv(v[0]);
		glVertex3fv(v[1]);
		glVertex3fv(v[2]);
	glEnd();
	}

	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);

	// Flush drawing commands
	glFlush();
	}





void CreateChild2(HWND hParent, HINSTANCE hInst)
	{
	WNDCLASS        wc;                     // Windows class structure

	// Register Window style
	wc.style                = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
	wc.lpfnWndProc          = (WNDPROC) WndProc2;
	wc.cbClsExtra           = 0;
	wc.cbWndExtra           = 0;
	wc.hInstance            = hInst;
	wc.hIcon                = NULL;
	wc.hCursor              = LoadCursor(NULL, IDC_ARROW);

	// No need for background brush for OpenGL window
	wc.hbrBackground        = NULL;

	wc.lpszMenuName         = NULL;
	wc.lpszClassName        = "Child2Class";

	// Register the window class
	RegisterClass(&wc);


	// Create the main application window
	hChild2 = CreateWindow(
				wc.lpszClassName,
				"Child2",

				// OpenGL requires WS_CLIPCHILDREN and WS_CLIPSIBLINGS
				WS_CHILD | WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN | WS_CLIPSIBLINGS,

				// Window position and size
				300, 10,
				250, 250,
				hParent,
				NULL,
				hInst,
				NULL);

  	// Display the window
	ShowWindow(hChild2,SW_SHOW);
	UpdateWindow(hChild2);
   }

// Window procedure, handles all messages for this program
LRESULT CALLBACK WndProc2(       HWND    hWnd,
							UINT    message,
							WPARAM  wParam,
							LPARAM  lParam)
	{
	static HDC hDC2;                 // Private GDI Device context

	switch (message)
		{
		// Window creation, setup for OpenGL
		case WM_CREATE:
			// Store the device context
			hDC2 = GetDC(hWnd);

			// Select the pixel format
			SetDCPixelFormat(hDC2);

			// Create the rendering context and make it current
			hRC2 = wglCreateContext(hDC2);
			wglMakeCurrent(hDC2, hRC2);
			SetupRC2();

         SetTimer(hWnd,102,1000,NULL);

			break;

      case WM_TIMER:
      	InvalidateRect(hWnd,NULL,FALSE);
      	break;

		// Window is being destroyed, cleanup
		case WM_DESTROY:
			// Deselect the current rendering context and delete it
			wglMakeCurrent(NULL,NULL);
			wglDeleteContext(hRC2);
			break;

		// Window is resized.
		case WM_SIZE:
			// Call our function which modifies the clipping
			// volume and viewport
 			wglMakeCurrent(hDC2, hRC2);
			ChangeSize(LOWORD(lParam), HIWORD(lParam));
			wglMakeCurrent(NULL, NULL);
			break;


		// The painting function.  This message sent by Windows
		// whenever the screen needs updating.
		case WM_PAINT:
			{
         RECT rect;
         GetClientRect(hWnd,&rect);

  			wglMakeCurrent(hDC2, hRC2);

			// Call OpenGL drawing code
     		// Rotate and translate the coordinate system
			// Note this will be cumulative

			glRotatef(5.0f, 1.0f, 0.0f, 0.0f);
			glRotatef(5.0f, 0.0f, 1.0f, 0.0f);

			RenderScene2();

			//glFlush();

			// Bring image to front
			SwapBuffers(hDC2);

  			wglMakeCurrent(NULL, NULL);

			// Do not validate, forcing a continuous repaint
         ValidateRect(hWnd,&rect);
			}
			break;

		// Windows is telling the application that it may modify
		// the system palette.  This message in essance asks the
		// application for a new palette.
		case WM_QUERYNEWPALETTE:
			// If the palette was created.
			if(hPalette)
				{
				int nRet;

				// Selects the palette into the current device context
				SelectPalette(hDC2, hPalette, FALSE);

				// Map entries from the currently selected palette to
				// the system palette.  The return value is the number
				// of palette entries modified.
				nRet = RealizePalette(hDC2);

				// Validate window contents
				ValidateRect(hWnd,FALSE);

				return nRet;
				}
			break;

      case WM_ERASEBKGND:
      	return FALSE;
         break;

		// This window may set the palette, even though it is not the
		// currently active window.
		case WM_PALETTECHANGED:
			// Don't do anything if the palette does not exist, or if
			// this is the window that changed the palette.
			if((hPalette != NULL) && ((HWND)wParam != hWnd))
				{
				// Select the palette into the device context
				SelectPalette(hDC2,hPalette,FALSE);

				// Map entries to system palette
				RealizePalette(hDC2);

				// Remap the current colors to the newly realized palette
				UpdateColors(hDC2);
				return 0;
				}
			break;

	default:   // Passes it on if unproccessed
	    return (DefWindowProc(hWnd, message, wParam, lParam));

	}

    return (0L);
	}

