// oglviewView.cpp : implementation of the COglviewView class
//

#include "stdafx.h"
#include "oglview.h"

#include "oglviewDoc.h"
#include "oglviewView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


// Forward declarations
void GLResize(GLsizei w, GLsizei h);
void GLRenderScene(void *pData);
void GLSetupRC(void *pData);

// Easier way to specify color for Windows bigots
#define glRGB(x, y, z)	glColor3ub((GLubyte)x, (GLubyte)y, (GLubyte)z)


HWND		ViewWindow;		/* View window */
HPALETTE	ViewPalette;		/* Color palette (if necessary) */
HDC		ViewDC;			/* Drawing context */
HGLRC		ViewRC;			/* OpenGL rendering context */


BITMAPINFO	*BitmapInfo = NULL;	/* Current bitmap information */
void		*BitmapBits = NULL;	/* Current bitmap pixel bits */

// May not need all of these
void			DisplayErrorMessage(char *, ...);
void			MakePalette(int);
LRESULT CALLBACK	ViewProc(HWND, UINT, WPARAM, LPARAM);
void			OpenBitmapFile(char *);
void			SaveBitmapFile(void);
void			PrintBitmap(void);
void			RepaintWindow(RECT *);


/////////////////////////////////////////////////////////////////////////////
// COglviewView

IMPLEMENT_DYNCREATE(COglviewView, CView)

BEGIN_MESSAGE_MAP(COglviewView, CView)
	//{{AFX_MSG_MAP(COglviewView)
	ON_WM_SIZE()
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_ERASEBKGND()
	ON_WM_PALETTECHANGED()
	ON_WM_QUERYNEWPALETTE()
	ON_COMMAND(ID_FILE_OPEN, OnFileOpen)
	ON_COMMAND(ID_FILE_PRINT, OnFilePrint)
	ON_COMMAND(ID_FILE_SAVE_AS, OnFileSaveAs)
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// COglviewView construction/destruction

COglviewView::COglviewView()
{
	// TODO: add construction code here

}

COglviewView::~COglviewView()
{
}

BOOL COglviewView::PreCreateWindow(CREATESTRUCT& cs)
{
	// Add Window styles required for OpenGL before window is created
	cs.style |= (WS_CLIPCHILDREN | WS_CLIPSIBLINGS | CS_OWNDC);

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// COglviewView drawing

void COglviewView::OnDraw(CDC* pDC)
{
	// Make the rendering context current
	wglMakeCurrent(m_hDC,m_hRC);

	// Call our external OpenGL code
	GLRenderScene(NULL);
	
	// Swap our scene to the front
	SwapBuffers(m_hDC);

	// Allow other rendering contexts to co-exist
	wglMakeCurrent(m_hDC,NULL);
}

/////////////////////////////////////////////////////////////////////////////
// COglviewView printing

BOOL COglviewView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void COglviewView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void COglviewView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// COglviewView diagnostics

#ifdef _DEBUG
void COglviewView::AssertValid() const
{
	CView::AssertValid();
}

void COglviewView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

COglviewDoc* COglviewView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(COglviewDoc)));
	return (COglviewDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// COglviewView message handlers

void COglviewView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);
}

int COglviewView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	int nPixelFormat;					// Pixel format index
	m_hDC = ::GetDC(m_hWnd);			// Get the Device context

	static PIXELFORMATDESCRIPTOR pfd = {
		sizeof(PIXELFORMATDESCRIPTOR),	// Size of this structure
		1,								// Version of this structure	
		PFD_DRAW_TO_WINDOW |			// Draw to Window (not to bitmap)
		PFD_SUPPORT_OPENGL |			// Support OpenGL calls in window
		PFD_DOUBLEBUFFER,				// Double buffered mode
		PFD_TYPE_RGBA,					// RGBA Color mode
		24,								// Want 24bit color 
		0,0,0,0,0,0,					// Not used to select mode
		0,0,							// Not used to select mode
		0,0,0,0,0,						// Not used to select mode
		32,								// Size of depth buffer
		0,								// Not used to select mode
		0,								// Not used to select mode
		PFD_MAIN_PLANE,					// Draw in main plane
		0,								// Not used to select mode
		0,0,0 };						// Not used to select mode

	// Choose a pixel format that best matches that described in pfd
	nPixelFormat = ChoosePixelFormat(m_hDC, &pfd);

	// Set the pixel format for the device context
	VERIFY(SetPixelFormat(m_hDC, nPixelFormat, &pfd));

	// Create the rendering context
	m_hRC = wglCreateContext(m_hDC);

	// Make the rendering context current, perform initialization, then
	// deselect it
	VERIFY(wglMakeCurrent(m_hDC,m_hRC));

	// Create the palette if needed
	InitializePalette();

	ViewWindow = this->m_hWnd;		/* View window */
	ViewPalette = (HPALETTE)m_GLPalette;		/* Color palette (if necessary) */
	ViewDC = m_hDC;			/* Drawing context */
	ViewRC = m_hRC;			/* OpenGL rendering context */


	return 0;
}

void COglviewView::OnDestroy() 
{
	CView::OnDestroy();

	// Clean up rendering context stuff
	wglMakeCurrent(NULL,NULL);
	wglDeleteContext(m_hRC);
	::ReleaseDC(m_hWnd,m_hDC);
}

BOOL COglviewView::OnEraseBkgnd(CDC* pDC) 
{
	return FALSE;
}

void COglviewView::OnPaletteChanged(CWnd* pFocusWnd) 
{
	if(((HPALETTE)m_GLPalette != NULL) && (pFocusWnd != this))
		{
		// Select the palette into the device context
		SelectPalette(m_hDC,(HPALETTE)m_GLPalette,FALSE);

		// Map entries to system palette
		RealizePalette(m_hDC);
				
		// Remap the current colors to the newly realized palette
		UpdateColors(m_hDC);
		return;
		}

	CView::OnPaletteChanged(pFocusWnd);
}

BOOL COglviewView::OnQueryNewPalette() 
{
	// If the palette was created.
	if((HPALETTE)m_GLPalette)
		{
		int nRet;

		// Selects the palette into the current device context
		SelectPalette(m_hDC, (HPALETTE)m_GLPalette, FALSE);

		// Map entries from the currently selected palette to
		// the system palette.  The return value is the number 
		// of palette entries modified.
		nRet = RealizePalette(m_hDC);

		// Repaint, forces remap of palette in current window
		InvalidateRect(NULL,FALSE);

		return nRet;
		}

	return CView::OnQueryNewPalette();
}


// Initializes the CPalette object
void COglviewView::InitializePalette(void)
	{
	PIXELFORMATDESCRIPTOR pfd;	// Pixel Format Descriptor
	LOGPALETTE *pPal;			// Pointer to memory for logical palette
	int nPixelFormat;			// Pixel format index
	int nColors;				// Number of entries in palette
	int i;						// Counting variable
	BYTE RedRange,GreenRange,BlueRange;
								// Range for each color entry (7,7,and 3)

	// Get the pixel format index and retrieve the pixel format description
	nPixelFormat = GetPixelFormat(m_hDC);
	DescribePixelFormat(m_hDC, nPixelFormat, sizeof(PIXELFORMATDESCRIPTOR), &pfd);

	// Does this pixel format require a palette?  If not, do not create a
	// palette and just return NULL
	if(!(pfd.dwFlags & PFD_NEED_PALETTE))
		return;

	// Number of entries in palette.  8 bits yeilds 256 entries
	nColors = 1 << pfd.cColorBits;	

	// Allocate space for a logical palette structure plus all the palette entries
	pPal = (LOGPALETTE*)malloc(sizeof(LOGPALETTE) +nColors*sizeof(PALETTEENTRY));

	// Fill in palette header 
	pPal->palVersion = 0x300;		// Windows 3.0
	pPal->palNumEntries = nColors; // table size

	// Build mask of all 1's.  This creates a number represented by having
	// the low order x bits set, where x = pfd.cRedBits, pfd.cGreenBits, and
	// pfd.cBlueBits.  
	RedRange = (1 << pfd.cRedBits) -1;
	GreenRange = (1 << pfd.cGreenBits) - 1;
	BlueRange = (1 << pfd.cBlueBits) -1;

	// Loop through all the palette entries
	for(i = 0; i < nColors; i++)
		{
		// Fill in the 8-bit equivalents for each component
		pPal->palPalEntry[i].peRed = (i >> pfd.cRedShift) & RedRange;
		pPal->palPalEntry[i].peRed = (unsigned char)(
			(double) pPal->palPalEntry[i].peRed * 255.0 / RedRange);

		pPal->palPalEntry[i].peGreen = (i >> pfd.cGreenShift) & GreenRange;
		pPal->palPalEntry[i].peGreen = (unsigned char)(
			(double)pPal->palPalEntry[i].peGreen * 255.0 / GreenRange);

		pPal->palPalEntry[i].peBlue = (i >> pfd.cBlueShift) & BlueRange;
		pPal->palPalEntry[i].peBlue = (unsigned char)(
			(double)pPal->palPalEntry[i].peBlue * 255.0 / BlueRange);

		pPal->palPalEntry[i].peFlags = (unsigned char) NULL;
		}
		
	// Create the palette
	m_GLPalette.CreatePalette(pPal);

	// Go ahead and select and realize the palette for this device context
	SelectPalette(m_hDC,(HPALETTE)m_GLPalette,FALSE);
	RealizePalette(m_hDC);

	// Free the memory used for the logical palette structure
	free(pPal);
	}



//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
// Render your OpenGL Scene here.
void GLRenderScene(void *pData)
	{
	RECT rect;

    GetClientRect(ViewWindow, &rect);
	RepaintWindow(&rect);
	}




/*
 * 'OpenBitmapFile()' - Load a bitmap file from disk.
 *
 * A filename dialog is popped if 'filename' is a NULL or blank string.
 */

void
OpenBitmapFile(char *filename)		/* I - Bitmap file to open */
{
  char		title[256],		/* Title of file (no path info) */
		name[256],		/* Full pathname of file */
		directory[256];		/* Directory of file */
  OPENFILENAME	ofn;			/* Filename dialog structure */
  void		*bits;			/* Bitmap pixel bits */


 /*
  * Pop up a filename dialog if we don't have a filename...
  */

  if (filename == NULL ||
      filename[0] == '\0')
  {
   /*
    * Fill in the OPENFILENAME dialog info...
    */

    strcpy(directory, ".");
    strcpy(name, "");
    strcpy(title, "");

    memset(&ofn, 0, sizeof(ofn));

    ofn.lStructSize      = sizeof(ofn);
    ofn.hwndOwner        = ViewWindow;
    ofn.lpstrFilter      = "Bitmaps\0*.BMP\0\0";
    ofn.nFilterIndex     = 1;
    ofn.lpstrFile        = name;
    ofn.nMaxFile         = sizeof(name) - 1;
    ofn.lpstrFileTitle   = title;
    ofn.nMaxFileTitle    = sizeof(title) - 1;
    ofn.lpstrInitialDir  = directory;
    ofn.lpstrTitle       = "Open Bitmap File";
    ofn.Flags            = OFN_HIDEREADONLY | OFN_PATHMUSTEXIST |
                           OFN_NONETWORKBUTTON | OFN_FILEMUSTEXIST;

    if (!GetOpenFileName(&ofn))
      return;

    filename = name;
  };

 /*
  * Free the old bitmap if there is one...
  */

  if (BitmapInfo != NULL)
  {
    free(BitmapInfo);
    free(BitmapBits);
    BitmapInfo = NULL;
  };

 /*
  * Load the named bitmap from disk.  Display an error message if it
  * doesn't load...
  */

  bits = LoadDIBitmap(filename, &BitmapInfo);

  if (bits == NULL)
    DisplayErrorMessage("Could not open bitmap file \'%s\'!", filename);
  else
  {
    BitmapBits = ConvertRGB(BitmapInfo, bits);
    free(bits);

    if (BitmapBits == NULL)
    {
      DisplayErrorMessage("Could not convert bitmap to RGB!");
      free(BitmapInfo);
      BitmapInfo = NULL;
    };
  };

 /*
  * Resize the window and force a repaint...
  */

  SetWindowText(ViewWindow, filename);
  SetWindowPos(ViewWindow, HWND_TOP, 0, 0,
               BitmapInfo->bmiHeader.biWidth,
               BitmapInfo->bmiHeader.biHeight + 40,
	       SWP_NOMOVE);
}


/*
 * 'SaveBitmapFile()' - Save the currently displayed bitmap to disk.
 */

void
SaveBitmapFile(void)
{
  char		title[256],
		filename[256],
		directory[256];
  OPENFILENAME	ofn;
  void		*bits;
  BITMAPINFO	*info;


 /*
  * Grab the screen bitmap...
  */

  bits = ReadDIBitmap(&info);
  if (bits == NULL)
  {
    DisplayErrorMessage("Unable to get OpenGL bitmap from screen!");
    return;
  };

 /*
  * Pop up a filename dialog...
  */

  strcpy(directory, ".");
  strcpy(filename, "untitled.bmp");
  strcpy(title, "");

  memset(&ofn, 0, sizeof(ofn));

  ofn.lStructSize      = sizeof(ofn);
  ofn.hwndOwner        = ViewWindow;
  ofn.lpstrFilter      = "Bitmaps\0*.BMP\0\0";
  ofn.nFilterIndex     = 1;
  ofn.lpstrFile        = filename;
  ofn.nMaxFile         = sizeof(filename) - 1;
  ofn.lpstrFileTitle   = title;
  ofn.nMaxFileTitle    = sizeof(title) - 1;
  ofn.lpstrInitialDir  = directory;
  ofn.lpstrTitle       = "Save Bitmap File";
  ofn.Flags            = OFN_HIDEREADONLY | OFN_PATHMUSTEXIST |
                         OFN_NONETWORKBUTTON;

  if (GetSaveFileName(&ofn))
  {
   /*
    * Save the named bitmap to disk...
    */

    if (SaveDIBitmap(filename, info, bits))
      DisplayErrorMessage("Could not save to file \'%s\' -\n%s",
                          filename, strerror(errno));
  };

 /*
  * Free memory and return...
  */

  free(info);
  free(bits);
}


/*
 * 'PrintBitmap()' - Print the currently displayed bitmap.
 */

void
PrintBitmap(void)
{
  void		*bits;
  BITMAPINFO	*info;


 /*
  * Grab the screen bitmap...
  */

  bits = ReadDIBitmap(&info);
  if (bits == NULL)
  {
    DisplayErrorMessage("Unable to get OpenGL bitmap from screen!");
    return;
  };

 /*
  * Try printing it...
  */

  PrintDIBitmap(ViewWindow, info, bits);

 /*
  * Free memory and return...
  */

  free(info);
  free(bits);
}


/*
 * 'RepaintWindow()' - Redraw the client area with our bitmap...
 */

void
RepaintWindow(RECT        *rect)	/* I - Client area rectangle */
{
  GLint		xoffset,		/* X offset of image */
  		yoffset;		/* Y offset of image */
  GLint		xsize,			/* X size of scaled image */
		ysize;			/* Y size of scaled image */
  GLfloat	xscale,			/* Scaling in X direction */
		yscale;			/* Scaling in Y direction */


 /*
  * Reset the viewport and clear the window to white...
  */

  glViewport(0, 0, rect->right, rect->bottom);

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(0.0, rect->right - 1.0, 0.0, rect->bottom - 1.0, -1.0, 1.0);
  glMatrixMode(GL_MODELVIEW);

  glClearColor(1.0, 1.0, 1.0, 1.0);
  glClear(GL_COLOR_BUFFER_BIT);

 /*
  * If we have loaded a bitmap image, scale it to fit the window...
  */

  if (BitmapBits != NULL)
  {
    xsize = rect->right;
    ysize = BitmapInfo->bmiHeader.biHeight * xsize /
            BitmapInfo->bmiHeader.biWidth;
    if (ysize > rect->bottom)
    {
      ysize = rect->bottom;
      xsize = BitmapInfo->bmiHeader.biWidth * ysize /
              BitmapInfo->bmiHeader.biHeight;
    };

    xscale  = (float)xsize / (float)BitmapInfo->bmiHeader.biWidth;
    yscale  = (float)ysize / (float)BitmapInfo->bmiHeader.biHeight;

    xoffset = (rect->right - xsize) * 0.5;
    yoffset = (rect->bottom - ysize) * 0.5;

    glPixelStorei(GL_UNPACK_ALIGNMENT, 4);
    glPixelZoom(xscale, yscale);
    glRasterPos2i(xoffset, yoffset);
    glDrawPixels(BitmapInfo->bmiHeader.biWidth,
                 BitmapInfo->bmiHeader.biHeight,
		 GL_RGB, GL_UNSIGNED_BYTE, BitmapBits);
  };

  glFinish();
}


/*
 * 'DisplayErrorMessage()' - Display an error message dialog.
 */

void
DisplayErrorMessage(char *format,	/* I - printf() style format string */
                    ...)		/* I - Other arguments as necessary */
{
  va_list	ap;			/* Argument pointer */
  char		s[1024];		/* Output string */


  if (format == NULL)
    return;

  va_start(ap, format);
  vsprintf(s, format, ap);
  va_end(ap);

  MessageBeep(MB_ICONEXCLAMATION);
  MessageBox(NULL, s, "Error", MB_OK | MB_ICONEXCLAMATION);
}

void COglviewView::OnFileOpen() 
	{
	OpenBitmapFile(NULL);
	}

void COglviewView::OnFilePrint() 
	{
    PrintBitmap();
	}


void COglviewView::OnFileSaveAs() 
	{
	SaveBitmapFile();
	}
