// GLPrint.c
// Demonstrates Printing with OpenGL the Old Fashioned way

#include <windows.h>            // Window defines
#include <gl\gl.h>              // OpenGL
#include <gl\glu.h>             // GLU library
#include <gl\glaux.h>                   // AUX library objects
#include "resource.h"           // About box resource identifiers.

#define glRGB(x, y, z)  glColor3ub((GLubyte)x, (GLubyte)y, (GLubyte)z)

void RenderToPrinter(HWND hParent);

HPALETTE hPalette = NULL;

// Application name and instance storeage
static LPCTSTR lpszAppName = "Printed OpenGL";
static HINSTANCE hInstance;


// Light values and coordinates
GLfloat  lightPos[] = { -50.0f, 100.0f, 150.0f, 0.0f };
GLfloat  specular[] = { 1.0f, 1.0f, 1.0f, 1.0f};
GLfloat  specref[] =  { 1.0f, 1.0f, 1.0f, 1.0f };
GLfloat  ambientLight[] = { 0.3f, 0.3f, 0.3f, 1.0f};
GLfloat  diffuseLight[] = { 0.8f, 0.8f, 0.8f, 1.0f };


// Declaration for Window procedure
LRESULT CALLBACK WndProc(   HWND    hWnd,
							UINT    message,
							WPARAM  wParam,
							LPARAM  lParam);

// Dialog procedure for about box
BOOL APIENTRY AboutDlgProc (HWND hDlg, UINT message, UINT wParam, LONG lParam);

// Set Pixel Format function - forward declaration
void SetDCPixelFormat(HDC hDC);


// Change viewing volume and viewport.  Called when window is resized
void ChangeSize(GLsizei w, GLsizei h)
	{
	GLfloat nRange = 80.0f;

	// Prevent a divide by zero
	if(h == 0)
		h = 1;

	if(w == 0)
		w = 1;

	// Set Viewport to window dimensions
    glViewport(0, 0, w, h);

	// Reset coordinate system
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	// Establish clipping volume (left, right, bottom, top, near, far)
    if (w <= h) 
		glOrtho (-nRange, nRange, -nRange*h/w, nRange*h/w, -nRange, nRange);
    else 
		glOrtho (-nRange*w/h, nRange*w/h, -nRange, nRange, -nRange, nRange);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	// Specify new position and direction in rotated coords.
	glLightfv(GL_LIGHT0,GL_POSITION,lightPos);
	}



// This function does any needed initialization on the rendering
// context.  Here it sets up and initializes the lighting for
// the scene.
void SetupRC()
	{
	glEnable(GL_DEPTH_TEST);        // Hidden surface removal
	glFrontFace(GL_CW);             // Counter clock-wise polygons face out
	glEnable(GL_CULL_FACE);         // Do not try to display the back sides
	glShadeModel(GL_SMOOTH);        // Smooth shading
	
	// Enable lighting
	glEnable(GL_LIGHTING);

	// Setup and enable light 0
	// Supply a slight ambient light so the objects can be seen
	// The light is composed of just a diffuse and specular components
	glLightfv(GL_LIGHT0,GL_AMBIENT,ambientLight);
	glLightfv(GL_LIGHT0,GL_DIFFUSE,diffuseLight);
	glLightfv(GL_LIGHT0,GL_SPECULAR,specular);
	glLightfv(GL_LIGHT0,GL_POSITION,lightPos);

	
	// Enable this light in particular
	glEnable(GL_LIGHT0);

	// Enable color tracking
	glEnable(GL_COLOR_MATERIAL);
	
	// Set Material properties to follow glColor values
	glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);

	// All materials hereafter have full specular reflectivity
	// with a high shine
	glMaterialfv(GL_FRONT, GL_SPECULAR,specref);
	glMateriali(GL_FRONT, GL_SHININESS,128);


	glEnable(GL_NORMALIZE);
	glEnable(GL_AUTO_NORMAL);

	// White background (works better on a printer!)
	glClearColor(1.0f, 1.0f, 1.0f, 1.0f );
	}




// Called to draw scene
void RenderScene(void)
	{
	// Clear the window with current clearing color
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glRGB(255, 0, 0);
	
	glPushMatrix();
		glTranslatef(-40.0f, 0.0f, 0.0f);
		auxSolidTeapot(30.0f);
	
		glTranslatef(100.0f, 0.0f, 0.0f);
		glRotatef(90.0f, 1.0f, 0.0f, 0.0f);
		glRotatef(90.0f, 0.0f, 1.0f, 0.0f);
		auxSolidTeapot(30.0f);
	glPopMatrix();

	// Flush drawing commands
	glFlush();
	}


// Select the pixel format for a given device context
void SetDCPixelFormat(HDC hDC)
	{
	int nPixelFormat;

	static PIXELFORMATDESCRIPTOR pfd = {
		sizeof(PIXELFORMATDESCRIPTOR),  // Size of this structure
		1,                                                              // Version of this structure    
		PFD_DRAW_TO_WINDOW |                    // Draw to Window (not to bitmap)
		PFD_SUPPORT_OPENGL |                                    // Support OpenGL calls in window
		PFD_SUPPORT_GDI,                                                // Allow GDI drawing in window too
		PFD_TYPE_RGBA,                          // RGBA Color mode
		24,                                     // Want 24bit color 
		0,0,0,0,0,0,                            // Not used to select mode
		0,0,                                    // Not used to select mode
		0,0,0,0,0,                              // Not used to select mode
		16,                                     // Size of depth buffer
		0,                                      // Not used to select mode
		0,                                      // Not used to select mode
		PFD_MAIN_PLANE,                         // Draw in main plane
		0,                                      // Not used to select mode
		0,0,0 };                                // Not used to select mode

	// Choose a pixel format that best matches that described in pfd
	nPixelFormat = ChoosePixelFormat(hDC, &pfd);

	// Set the pixel format for the device context
	SetPixelFormat(hDC, nPixelFormat, &pfd);
	}



// If necessary, creates a 3-3-2 palette for the device context listed.
HPALETTE GetOpenGLPalette(HDC hDC)
	{
	HPALETTE hRetPal = NULL;        // Handle to palette to be created
	PIXELFORMATDESCRIPTOR pfd;      // Pixel Format Descriptor
	LOGPALETTE *pPal;                       // Pointer to memory for logical palette
	int nPixelFormat;                       // Pixel format index
	int nColors;                            // Number of entries in palette
	int i;                                          // Counting variable
	BYTE RedRange,GreenRange,BlueRange;
								// Range for each color entry (7,7,and 3)


	// Get the pixel format index and retrieve the pixel format description
	nPixelFormat = GetPixelFormat(hDC);
	DescribePixelFormat(hDC, nPixelFormat, sizeof(PIXELFORMATDESCRIPTOR), &pfd);

	// Does this pixel format require a palette?  If not, do not create a
	// palette and just return NULL
	if(!(pfd.dwFlags & PFD_NEED_PALETTE))
		return NULL;

	// Number of entries in palette.  8 bits yeilds 256 entries
	nColors = 1 << pfd.cColorBits;  

	// Allocate space for a logical palette structure plus all the palette entries
	pPal = (LOGPALETTE*)malloc(sizeof(LOGPALETTE) +nColors*sizeof(PALETTEENTRY));

	// Fill in palette header 
	pPal->palVersion = 0x300;               // Windows 3.0
	pPal->palNumEntries = nColors; // table size

	// Build mask of all 1's.  This creates a number represented by having
	// the low order x bits set, where x = pfd.cRedBits, pfd.cGreenBits, and
	// pfd.cBlueBits.  
	RedRange = (1 << pfd.cRedBits) -1;
	GreenRange = (1 << pfd.cGreenBits) - 1;
	BlueRange = (1 << pfd.cBlueBits) -1;

	// Loop through all the palette entries
	for(i = 0; i < nColors; i++)
		{
		// Fill in the 8-bit equivalents for each component
		pPal->palPalEntry[i].peRed = (i >> pfd.cRedShift) & RedRange;
		pPal->palPalEntry[i].peRed = (unsigned char)(
			(double) pPal->palPalEntry[i].peRed * 255.0 / RedRange);

		pPal->palPalEntry[i].peGreen = (i >> pfd.cGreenShift) & GreenRange;
		pPal->palPalEntry[i].peGreen = (unsigned char)(
			(double)pPal->palPalEntry[i].peGreen * 255.0 / GreenRange);

		pPal->palPalEntry[i].peBlue = (i >> pfd.cBlueShift) & BlueRange;
		pPal->palPalEntry[i].peBlue = (unsigned char)(
			(double)pPal->palPalEntry[i].peBlue * 255.0 / BlueRange);

		pPal->palPalEntry[i].peFlags = (unsigned char) NULL;
		}
		

	// Create the palette
	hRetPal = CreatePalette(pPal);

	// Go ahead and select and realize the palette for this device context
	SelectPalette(hDC,hRetPal,FALSE);
	RealizePalette(hDC);

	// Free the memory used for the logical palette structure
	free(pPal);

	// Return the handle to the new palette
	return hRetPal;
	}


// Entry point of all Windows programs
int APIENTRY WinMain(   HINSTANCE       hInst,
						HINSTANCE       hPrevInstance,
						LPSTR           lpCmdLine,
						int                     nCmdShow)
	{
	MSG                     msg;            // Windows message structure
	WNDCLASS        wc;                     // Windows class structure
	HWND            hWnd;           // Storeage for window handle

	hInstance = hInst;

	// Register Window style
	wc.style                        = CS_HREDRAW | CS_VREDRAW;
	wc.lpfnWndProc          = (WNDPROC) WndProc;
	wc.cbClsExtra           = 0;
	wc.cbWndExtra           = 0;
	wc.hInstance            = hInstance;
	wc.hIcon                        = NULL;
	wc.hCursor                      = LoadCursor(NULL, IDC_ARROW);
	
	// No need for background brush for OpenGL window
	wc.hbrBackground        = NULL;         
	
	wc.lpszMenuName         = MAKEINTRESOURCE(IDR_MENU);
	wc.lpszClassName        = lpszAppName;

	// Register the window class
	if(RegisterClass(&wc) == 0)
		return FALSE;


	// Create the main application window
	hWnd = CreateWindow(
				lpszAppName,
				lpszAppName,
				
				// OpenGL requires WS_CLIPCHILDREN and WS_CLIPSIBLINGS
				WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN | WS_CLIPSIBLINGS,
	
				// Window position and size
				10, 10,
				300, 300,
				NULL,
				NULL,
				hInstance,
				NULL);

	// If window was not created, quit
	if(hWnd == NULL)
		return FALSE;


	// Display the window
	ShowWindow(hWnd,SW_SHOW);
	UpdateWindow(hWnd);

	// Process application messages until the application closes
	while( GetMessage(&msg, NULL, 0, 0))
		{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
		}

	return msg.wParam;
	}




// Window procedure, handles all messages for this program
LRESULT CALLBACK WndProc(       HWND    hWnd,
							UINT    message,
							WPARAM  wParam,
							LPARAM  lParam)
	{
	static HGLRC hRC;               // Permenant Rendering context
	static HDC hDC;                 // Private GDI Device context

	switch (message)
		{
		// Window creation, setup for OpenGL
		case WM_CREATE:
			// Store the device context
			hDC = GetDC(hWnd);              

			// Select the pixel format
			SetDCPixelFormat(hDC);          

			hPalette = GetOpenGLPalette(hDC);

			// Create the rendering context and make it current
			hRC = wglCreateContext(hDC);
			wglMakeCurrent(hDC, hRC);
			SetupRC();

			break;

		// Window is being destroyed, cleanup
		case WM_DESTROY:
			// Deselect the current rendering context and delete it
			wglMakeCurrent(hDC,NULL);
			wglDeleteContext(hRC);

			if(hPalette != NULL)
				DeleteObject(hPalette);

			// Tell the application to terminate after the window
			// is gone.
			PostQuitMessage(0);
			break;

		// Window is resized.
		case WM_SIZE:
			// Call our function which modifies the clipping
			// volume and viewport
			ChangeSize(LOWORD(lParam), HIWORD(lParam));
			break;


		// The painting function.  This message sent by Windows 
		// whenever the screen needs updating.
		case WM_PAINT:
			{
			// Call OpenGL drawing code
			RenderScene();

			// Validate the newly painted client area
			ValidateRect(hWnd,NULL);
			}
			break;

		// Windows is telling the application that it may modify
		// the system palette.  This message in essance asks the 
		// application for a new palette.
		case WM_QUERYNEWPALETTE:
			// If the palette was created.
			if(hPalette)
				{
				int nRet;

				// Selects the palette into the current device context
				SelectPalette(hDC, hPalette, FALSE);

				// Map entries from the currently selected palette to
				// the system palette.  The return value is the number 
				// of palette entries modified.
				nRet = RealizePalette(hDC);

				// Repaint, forces remap of palette in current window
				InvalidateRect(hWnd,NULL,FALSE);

				return nRet;
				}
			break;

	
		// This window may set the palette, even though it is not the 
		// currently active window.
		case WM_PALETTECHANGED:
			// Don't do anything if the palette does not exist, or if
			// this is the window that changed the palette.
			if((hPalette != NULL) && ((HWND)wParam != hWnd))
				{
				// Select the palette into the device context
				SelectPalette(hDC,hPalette,FALSE);

				// Map entries to system palette
				RealizePalette(hDC);
				
				// Remap the current colors to the newly realized palette
				UpdateColors(hDC);
				return 0;
				}
			break;


		// A menu command
		case WM_COMMAND:
			{
			switch(LOWORD(wParam))
				{
				// Exit the program
				case ID_FILE_EXIT:
					DestroyWindow(hWnd);
					break;

				// Display the about box
				case ID_HELP_ABOUT:
					DialogBox (hInstance,
						MAKEINTRESOURCE(IDD_DIALOG_ABOUT),
						hWnd,
						AboutDlgProc);
					break;

				case ID_PRINT:
					RenderToPrinter(hWnd);          // Prints the image via another RC
					wglMakeCurrent(hDC, hRC);       // Makes the current RC current again
					break;
				}
			}
			break;


	default:   // Passes it on if unproccessed
	    return (DefWindowProc(hWnd, message, wParam, lParam));

	}

    return (0L);
	}




// Dialog procedure.
BOOL APIENTRY AboutDlgProc (HWND hDlg, UINT message, UINT wParam, LONG lParam)
	{
	
    switch (message)
	{
		// Initialize the dialog box
	    case WM_INITDIALOG:
			{
			int i;
			GLenum glError;

			// glGetString demo
			SetDlgItemText(hDlg,IDC_OPENGL_VENDOR,glGetString(GL_VENDOR));
			SetDlgItemText(hDlg,IDC_OPENGL_RENDERER,glGetString(GL_RENDERER));
			SetDlgItemText(hDlg,IDC_OPENGL_VERSION,glGetString(GL_VERSION));
			SetDlgItemText(hDlg,IDC_OPENGL_EXTENSIONS,glGetString(GL_EXTENSIONS));

			// gluGetString demo
			SetDlgItemText(hDlg,IDC_GLU_VERSION,gluGetString(GLU_VERSION));
			SetDlgItemText(hDlg,IDC_GLU_EXTENSIONS,gluGetString(GLU_EXTENSIONS));


			// Display any recent error messages
			i = 0;
			do {
				glError = glGetError();
				SetDlgItemText(hDlg,IDC_ERROR1+i,gluErrorString(glError));
				i++;
				}
			while(i < 6 && glError != GL_NO_ERROR);


			return (TRUE);
			}
			break;

		// Process command messages
	    case WM_COMMAND:      
			{
			// Validate and Make the changes
			if(LOWORD(wParam) == IDOK)
				EndDialog(hDlg,TRUE);
		    }
			break;

		// Closed from sysbox
		case WM_CLOSE:
			EndDialog(hDlg,TRUE);
			break;
		}

	return FALSE;
	}


// Render the Image to a printer, this should be a color printer
// capable of more than 4bits of color depth.
void RenderToPrinter(HWND hParent)
	{
	short cxPage, cyPage;   // Dimensions of Printed Page
	BOOL bMetaFile = FALSE;	// MetaFile capabilities exist
	OSVERSIONINFO osVersion;// OS identifier
	HGLRC hRC;				// Rendering context for printer
	PRINTDLG printData;     // Printing data
	int nPixelFormat;       // Pixel format
	DOCINFO docInfo;        // Document Info

	static PIXELFORMATDESCRIPTOR pfd = {
		sizeof(PIXELFORMATDESCRIPTOR),  // Size of this structure
		1,                                                              // Version of this structure    
		PFD_DRAW_TO_WINDOW |                    // Draw to Window (not to bitmap)
		PFD_SUPPORT_OPENGL |                                    // Support OpenGL calls in window
		PFD_SUPPORT_GDI,                                                // Allow GDI drawing in window too
		PFD_TYPE_RGBA,                          // RGBA Color mode
		16,                                     // Want 16bit color 
		0,0,0,0,0,0,                            // Not used to select mode
		0,0,                                    // Not used to select mode
		0,0,0,0,0,                              // Not used to select mode
		16,                                     // Size of depth buffer
		0,                                      // Not used to select mode
		0,                                      // Not used to select mode
		PFD_MAIN_PLANE,                         // Draw in main plane
		0,                                      // Not used to select mode
		0,0,0 };                                // Not used to select mode

	memset(&printData,0,sizeof(PRINTDLG));

	printData.lStructSize = sizeof(PRINTDLG);
	printData.hwndOwner = hParent;
	printData.hDevMode = NULL;
	printData.hDevNames = NULL;
	printData.Flags = PD_RETURNDC | PD_ALLPAGES;

	if(!PrintDlg(&printData))
		return;

	// Initialize DocInfo structure
	docInfo.cbSize = sizeof(DOCINFO);
	docInfo.lpszDocName = "OpenGL Rendering";
	docInfo.lpszOutput = NULL;


	// Check for Metafile capabilities (NT 4.0 or greater, no Windows95)
	GetVersionEx(&osVersion);

	// If NT AND version 4.x or greater
	if(osVersion.dwPlatformId == VER_PLATFORM_WIN32_NT &&
		osVersion.dwMajorVersion >= 4)
		bMetaFile = TRUE;

	// If no metafile capabilies, create the rendering context now
	if(!bMetaFile)
		{
		// Now we have a printer device context, create a pixel format,
		// and rendering context

		// Choose a pixel format that best matches that described in pfd
		nPixelFormat = ChoosePixelFormat(printData.hDC, &pfd);

		if(nPixelFormat == 0)
			{
			MessageBox(hParent,"Cannot choose a pixel format for this printer!",NULL,
					MB_OK | MB_ICONSTOP);
			return;
			}

		// Set the pixel format for the device context
		if(!SetPixelFormat(printData.hDC, nPixelFormat, &pfd))
			{
			MessageBox(hParent,"Error Setting pixel format for this printer!",NULL,
					MB_OK | MB_ICONSTOP);
			return;
			}


		// Create the Rendering context and execute the OpenGL Commands
		hRC = wglCreateContext(printData.hDC);
		wglMakeCurrent(printData.hDC, hRC);
		}


	// Start the document and page
	StartDoc(printData.hDC, &docInfo);
	StartPage(printData.hDC);

	// If we are Metafile enabled, then create rendering context here
	if(bMetaFile)
		{
		// Now we have a printer device context, create a pixel format,
		// and rendering context

		// Choose a pixel format that best matches that described in pfd
		nPixelFormat = ChoosePixelFormat(printData.hDC, &pfd);

		if(nPixelFormat == 0)
			{
			MessageBox(hParent,"Cannot choose a pixel format for this printer!",NULL,
					MB_OK | MB_ICONSTOP);
			return;
			}

		// Set the pixel format for the device context
		if(!SetPixelFormat(printData.hDC, nPixelFormat, &pfd))
			{
			MessageBox(hParent,"Error Setting pixel format for this printer!",NULL,
					MB_OK | MB_ICONSTOP);
			return;
			}


		// Create the Rendering context and execute the OpenGL Commands
		hRC = wglCreateContext(printData.hDC);
		wglMakeCurrent(printData.hDC, hRC);
		}


	// Setup GL
	SetupRC();

	// Get the dimensions of the page
	cxPage = GetDeviceCaps(printData.hDC, HORZRES);
	cyPage = GetDeviceCaps(printData.hDC, VERTRES);
	
	// Set size of viewing volume accordingly
	ChangeSize(cxPage, cyPage);

	// Render just like normally would
	RenderScene();

	// Signify page and document done
	EndPage(printData.hDC); // Finish writing to the page
	EndDoc(printData.hDC); // End the print job


	// Clean up the rendering context
	wglMakeCurrent(NULL,NULL);
	wglDeleteContext(hRC);

	// Delete the printer context when done with it
	DeleteDC(printData.hDC);
	}
