// Visteon.c
// Visteon Screen Saver

// Required Headers
#include <windows.h>
#include <scrnsave.h>
#include <gl\gl.h>
#include <gl\glu.h>

#include "glcode.h"

/////////////////////////////////////////////////////////////////////////////////
// GLOBAL Variables for this file
HGLRC hRC = (HGLRC) 0;
HDC hDC = (HDC) 0;
HPALETTE hPalette = (HPALETTE) 0;

UINT nTimerID = 101;



///////////////////////////////////////////////////////////////////////////////
// FUNCTION PROTOTYPES
void InitializeGLPalette(void);



// Process Messages for the main Window
LRESULT WINAPI ScreenSaverProc(HWND  hWnd,UINT  message, WPARAM  wParam, LPARAM  lParam)
	{
    HDC hdc;
    PAINTSTRUCT ps;
    static BOOL bInited = FALSE;
	int nPixelFormat;					// Pixel format index

	static PIXELFORMATDESCRIPTOR pfd = {
		sizeof(PIXELFORMATDESCRIPTOR),	// Size of this structure
		1,								// Version of this structure	
		PFD_DRAW_TO_WINDOW |			// Draw to Window (not to bitmap)
		PFD_SUPPORT_OPENGL |			// Support OpenGL calls in window
		PFD_DOUBLEBUFFER,   			// Double buffered mode
		PFD_TYPE_RGBA,					// RGBA Color mode
		24,								// Want 24bit color 
		0,0,0,0,0,0,					// Not used to select mode
		0,0,							// Not used to select mode
		0,0,0,0,0,						// Not used to select mode
		16,								// Size of depth buffer
		0,								// Not used to select mode
		0,								// Not used to select mode
		PFD_MAIN_PLANE,					// Draw in main plane
		0,								// Not used to select mode
		0,0,0 };						// Not used to select mode


    switch (message)
		{
		// This isn't really needed, but if they change the implementation later,
		// this will safeguard to make sure these styles are set.
		case WM_NCCREATE:
			{
			LPCREATESTRUCT lpcs;

			lpcs = (LPCREATESTRUCT) lParam;

			lpcs->style |= (WS_CLIPCHILDREN | WS_CLIPSIBLINGS | CS_OWNDC);

			return TRUE;
			}
			break;

        case WM_CREATE:
			hDC = GetDC(hWnd);

			// Choose a pixel format that best matches that described in pfd
			nPixelFormat = ChoosePixelFormat(hDC, &pfd);

			// Set the pixel format for the device context
			SetPixelFormat(hDC, nPixelFormat, &pfd);

			// Create the palette if needed
			InitializeGLPalette();

			// Create the rendering context
			hRC = wglCreateContext(hDC);

			// Make the rendering context current, perform initialization, then
			// deselect it
			wglMakeCurrent(hDC,hRC);
			
			GLSetupRC(hDC);
		
			// Set Timer
			SetTimer(hWnd, nTimerID,33,NULL); // About 30 fps (yeah, right)

			break;

        case WM_DESTROY:
			KillTimer(hWnd, nTimerID);

			// Clean up rendering context
			wglMakeCurrent(hDC, NULL);
			wglDeleteContext(hRC);
			ReleaseDC(hWnd,hDC);

            PostQuitMessage(0);
            break;

		case WM_TIMER:
			InvalidateRect(hWnd, NULL, FALSE);
			break;

		case WM_ERASEBKGND:
			return 1;
			break;

		case WM_SIZE:
			GLResize(LOWORD(lParam), HIWORD(lParam));
			break;


        case WM_PAINT:
            hdc = BeginPaint(hWnd, &ps);
	
			// Call our external OpenGL code
			GLRenderScene(NULL);
			
			// Swap our scene to the front
			SwapBuffers(hDC);

            EndPaint(hWnd, &ps);
            break;

        default:
            return DefScreenSaverProc(hWnd, message, wParam, lParam);
	    }
    return 0;
	}


////////////////////////////////////////////////////
// Configuration dialog
BOOL WINAPI ScreenSaverConfigureDialog (HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
	{
    switch (message) 
		{
	    case WM_INITDIALOG:
			return TRUE;
			break;


        case WM_COMMAND:
            switch (LOWORD(wParam)) 
				{
                case IDOK:
                    EndDialog(hDlg, TRUE);
                    break;

                default:
                    break;
	            }
    
	        return TRUE;
            break;

        default:
            return 0;
			break;
		}
    
	return 0;
	}



// No Special Needs for special controls, etc.
BOOL WINAPI RegisterDialogClasses(HANDLE hInst)
	{
	return TRUE;
	}



///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////


// Initializes the CPalette object
void InitializeGLPalette(void)
	{
	PIXELFORMATDESCRIPTOR pfd;	// Pixel Format Descriptor
	LOGPALETTE *pPal;			// Pointer to memory for logical palette
	int nPixelFormat;			// Pixel format index
	int nColors;				// Number of entries in palette
	int i;						// Counting variable
	BYTE RedRange,GreenRange,BlueRange;
								// Range for each color entry (7,7,and 3)

	// Get the pixel format index and retrieve the pixel format description
	nPixelFormat = GetPixelFormat(hDC);
	DescribePixelFormat(hDC, nPixelFormat, sizeof(PIXELFORMATDESCRIPTOR), &pfd);

	// Does this pixel format require a palette?  If not, do not create a
	// palette and just return NULL
	if(!(pfd.dwFlags & PFD_NEED_PALETTE))
		return;

	// Number of entries in palette.  8 bits yeilds 256 entries
	nColors = 1 << pfd.cColorBits;	

	// Allocate space for a logical palette structure plus all the palette entries
	pPal = (LOGPALETTE*)malloc(sizeof(LOGPALETTE) +nColors*sizeof(PALETTEENTRY));

	// Fill in palette header 
	pPal->palVersion = 0x300;		// Windows 3.0
	pPal->palNumEntries = nColors; // table size

	// Build mask of all 1's.  This creates a number represented by having
	// the low order x bits set, where x = pfd.cRedBits, pfd.cGreenBits, and
	// pfd.cBlueBits.  
	RedRange = (1 << pfd.cRedBits) -1;
	GreenRange = (1 << pfd.cGreenBits) - 1;
	BlueRange = (1 << pfd.cBlueBits) -1;

	// Loop through all the palette entries
	for(i = 0; i < nColors; i++)
		{
		// Fill in the 8-bit equivalents for each component
		pPal->palPalEntry[i].peRed = (i >> pfd.cRedShift) & RedRange;
		pPal->palPalEntry[i].peRed = (unsigned char)(
			(double) pPal->palPalEntry[i].peRed * 255.0 / RedRange);

		pPal->palPalEntry[i].peGreen = (i >> pfd.cGreenShift) & GreenRange;
		pPal->palPalEntry[i].peGreen = (unsigned char)(
			(double)pPal->palPalEntry[i].peGreen * 255.0 / GreenRange);

		pPal->palPalEntry[i].peBlue = (i >> pfd.cBlueShift) & BlueRange;
		pPal->palPalEntry[i].peBlue = (unsigned char)(
			(double)pPal->palPalEntry[i].peBlue * 255.0 / BlueRange);

		pPal->palPalEntry[i].peFlags = (unsigned char) NULL;
		}
		
	// Create the palette
	hPalette = CreatePalette(pPal);

	// Go ahead and select and realize the palette for this device context
	SelectPalette(hDC,hPalette,FALSE);
	RealizePalette(hDC);

	// Free the memory used for the logical palette structure
	free(pPal);
	}
