//----------------------------------------------------------------------------
//  Project Slstbolt
//  Waite Group Press
//  Copyright  1996. All Rights Reserved.
//
//  SUBSYSTEM:    Slstbolt Application
//  FILE:         slstblta.cpp
//  AUTHOR:       Richard S. Wright Jr.
//
//  OVERVIEW
//  ~~~~~~~~
//  Source file for implementation of TSlstboltApp (TApplication).
//
//----------------------------------------------------------------------------

#include <owl/pch.h>

#include <stdio.h>

#include "slstblta.h"
#include "slstbltw.h"                        // Definition of client class.
#include "slstblad.h"                        // Definition of about dialog.


//{{TSlstboltApp Implementation}}


//
// Build a response table for all messages/commands handled
// by the application.
//
DEFINE_RESPONSE_TABLE1(TSlstboltApp, TApplication)
//{{TSlstboltAppRSP_TBL_BEGIN}}
  EV_COMMAND(CM_HELPABOUT, CmHelpAbout),
//{{TSlstboltAppRSP_TBL_END}}
END_RESPONSE_TABLE;


//--------------------------------------------------------
// TSlstboltApp
//
TSlstboltApp::TSlstboltApp() : TApplication("Display List Bolt")
{

  // Common file file flags and filters for Open/Save As dialogs.  Filename and directory are
  // computed in the member functions CmFileOpen, and CmFileSaveAs.
  //
  FileData.Flags = OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT;
  FileData.SetFilter("All Files (*.*)|*.*|");
  FileData.DefExt = "txt";

  // INSERT>> Your constructor code here.
}


TSlstboltApp::~TSlstboltApp()
{
  // INSERT>> Your destructor code here.
}


//--------------------------------------------------------
// TSlstboltApp
// ~~~~~
// Application intialization.
//
void TSlstboltApp::InitMainWindow()
{
  if (nCmdShow != SW_HIDE)
    nCmdShow = (nCmdShow != SW_SHOWMINNOACTIVE) ? SW_SHOWNORMAL : nCmdShow;

  TSDIDecFrame* frame = new TSDIDecFrame(0, GetName(), 0, false);

  // Override the default window style for the main window.
  //
      frame->Attr.Style |= WS_BORDER | WS_CAPTION | WS_CLIPCHILDREN | WS_MAXIMIZEBOX | WS_MINIMIZEBOX | WS_SYSMENU | WS_THICKFRAME | WS_VISIBLE;
    frame->Attr.Style &= ~(WS_CHILD);

  // Assign ICON w/ this application.
  //
  frame->SetIcon(this, IDI_SDIAPPLICATION);
  frame->SetIconSm(this, IDI_SDIAPPLICATION);

  // Menu associated with window and accelerator table associated with table.
  //
  frame->AssignMenu(IDM_SDI);

  // Associate with the accelerator table.
  //
  frame->Attr.AccelTable = IDM_SDI;

  SetMainWindow(frame);

  frame->SetMenuDescr(TMenuDescr(IDM_SDI));
}



//--------------------------------------------------------
// TSlstboltApp
// ~~~~~~~~~~~
// Menu File New command
//
void TSlstboltApp::CmFileNew()
{
}


//--------------------------------------------------------
// TSlstboltApp
// ~~~~~~~~~~~
// Menu File Open command
//
void TSlstboltApp::CmFileOpen()
{
  // Display standard Open dialog box to select a file name.
  //
  *FileData.FileName = 0;

  TSlstboltWindow* client = TYPESAFE_DOWNCAST(GetMainWindow()->GetClientWindow(), TSlstboltWindow);     // Client window for the frame.
  if (client->CanClose())
    if (TFileOpenDialog(GetMainWindow(), FileData).Execute() == IDOK)
      OpenFile();
}


void TSlstboltApp::OpenFile(const char* fileName)
{
  if (fileName)
    strcpy(FileData.FileName, fileName);

}


//{{TSDIDecFrame Implementation}}


TSDIDecFrame::TSDIDecFrame(TWindow* parent, const char far* title, TWindow* clientWnd, bool trackMenuSelection, TModule* module)
:
  TDecoratedFrame(parent, title, !clientWnd ? new TSlstboltWindow(0, "") : clientWnd, trackMenuSelection, module)
{
  // INSERT>> Your constructor code here.

}


TSDIDecFrame::~TSDIDecFrame()
{
  // INSERT>> Your destructor code here.

}


//--------------------------------------------------------
// TSlstboltApp
// ~~~~~~~~~~~
// Menu Help About Slstbolt command
//
void TSlstboltApp::CmHelpAbout()
{
  // Show the modal dialog.
  //
  TSlstboltAboutDlg(MainWindow).Execute();
}



int OwlMain(int , char* [])
{
  TSlstboltApp   app;
  return app.Run();
}
