//----------------------------------------------------------------------------
//  Project Oglview
//  Waite Group Press
//  Copyright  1996. All Rights Reserved.
//
//  SUBSYSTEM:    Oglview Application
//  FILE:         oglvwapp.cpp
//  AUTHOR:       Richard S. Wright Jr.
//
//  OVERVIEW
//  ~~~~~~~~
//  Source file for implementation of TOglviewApp (TApplication).
//
//----------------------------------------------------------------------------

#include <owl/pch.h>

#include <stdio.h>

#include "oglvwapp.h"
#include "oglvwwnd.h"                        // Definition of client class.
#include "oglvwabd.h"                        // Definition of about dialog.


//{{TOglviewApp Implementation}}


//
// Build a response table for all messages/commands handled
// by the application.
//
DEFINE_RESPONSE_TABLE1(TOglviewApp, TApplication)
//{{TOglviewAppRSP_TBL_BEGIN}}
  EV_COMMAND(CM_FILEOPEN, CmFileOpen),
  EV_COMMAND(CM_HELPABOUT, CmHelpAbout),
//{{TOglviewAppRSP_TBL_END}}
END_RESPONSE_TABLE;


//--------------------------------------------------------
// TOglviewApp
//
TOglviewApp::TOglviewApp() : TApplication("OpenGL Bitmap Viewer")
{

  // Common file file flags and filters for Open/Save As dialogs.  Filename and directory are
  // computed in the member functions CmFileOpen, and CmFileSaveAs.
  //
  FileData.Flags = OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT;
  FileData.SetFilter("All Files (*.*)|*.*|");
  FileData.DefExt = "txt";

  // INSERT>> Your constructor code here.
}


TOglviewApp::~TOglviewApp()
{
  // INSERT>> Your destructor code here.
}


//--------------------------------------------------------
// TOglviewApp
// ~~~~~
// Application intialization.
//
void TOglviewApp::InitMainWindow()
{
  if (nCmdShow != SW_HIDE)
    nCmdShow = (nCmdShow != SW_SHOWMINNOACTIVE) ? SW_SHOWNORMAL : nCmdShow;

  TSDIDecFrame* frame = new TSDIDecFrame(0, GetName(), 0, false);

  // Assign ICON w/ this application.
  //
  frame->SetIcon(this, IDI_SDIAPPLICATION);
  frame->SetIconSm(this, IDI_SDIAPPLICATION);

  // Menu associated with window and accelerator table associated with table.
  //
  frame->AssignMenu(IDM_SDI);

  // Associate with the accelerator table.
  //
  frame->Attr.AccelTable = IDM_SDI;

  SetMainWindow(frame);

  frame->SetMenuDescr(TMenuDescr(IDM_SDI));
}



//--------------------------------------------------------
// TOglviewApp
// ~~~~~~~~~~~
// Menu File New command
//
void TOglviewApp::CmFileNew()
{
}


//--------------------------------------------------------
// TOglviewApp
// ~~~~~~~~~~~
// Menu File Open command
//
void TOglviewApp::CmFileOpen()
{
  // Display standard Open dialog box to select a file name.
  //
  *FileData.FileName = 0;

  TOglviewWindow* client = TYPESAFE_DOWNCAST(GetMainWindow()->GetClientWindow(), TOglviewWindow);     // Client window for the frame.
  if (client->CanClose())
    if (TFileOpenDialog(GetMainWindow(), FileData).Execute() == IDOK)
      OpenFile();
}


void TOglviewApp::OpenFile(const char* fileName)
{
  if (fileName)
    strcpy(FileData.FileName, fileName);

}


//{{TSDIDecFrame Implementation}}


TSDIDecFrame::TSDIDecFrame(TWindow* parent, const char far* title, TWindow* clientWnd, bool trackMenuSelection, TModule* module)
:
  TDecoratedFrame(parent, title, !clientWnd ? new TOglviewWindow(0, "") : clientWnd, trackMenuSelection, module)
{
  // INSERT>> Your constructor code here.

}


TSDIDecFrame::~TSDIDecFrame()
{
  // INSERT>> Your destructor code here.

}


//--------------------------------------------------------
// TOglviewApp
// ~~~~~~~~~~~
// Menu Help About Oglview command
//
void TOglviewApp::CmHelpAbout()
{
  // Show the modal dialog.
  //
  TOglviewAboutDlg(MainWindow).Execute();
}



int OwlMain(int , char* [])
{
  TOglviewApp   app;
  return app.Run();
}
