//----------------------------------------------------------------------------
//  Project Triangle
//  Waite Group Press
//  Copyright  1996. All Rights Reserved.
//
//  SUBSYSTEM:    Triangle Application
//  FILE:         trnglapp.cpp
//  AUTHOR:       Richard S. Wright Jr.
//
//  OVERVIEW
//  ~~~~~~~~
//  Source file for implementation of TTriangleApp (TApplication).
//
//----------------------------------------------------------------------------

#include <owl/pch.h>

#include <stdio.h>

#include "trnglapp.h"
#include "trnglwnd.h"                        // Definition of client class.
#include "trnglabd.h"                        // Definition of about dialog.


//{{TTriangleApp Implementation}}


//
// Build a response table for all messages/commands handled
// by the application.
//
DEFINE_RESPONSE_TABLE1(TTriangleApp, TApplication)
//{{TTriangleAppRSP_TBL_BEGIN}}
  EV_COMMAND(CM_FILENEW, CmFileNew),
  EV_COMMAND(CM_FILEOPEN, CmFileOpen),
  EV_COMMAND(CM_HELPABOUT, CmHelpAbout),
//{{TTriangleAppRSP_TBL_END}}
END_RESPONSE_TABLE;


//--------------------------------------------------------
// TTriangleApp
//
TTriangleApp::TTriangleApp() : TApplication("Triangle")
{

  // Common file file flags and filters for Open/Save As dialogs.  Filename and directory are
  // computed in the member functions CmFileOpen, and CmFileSaveAs.
  //
  FileData.Flags = OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT;
  FileData.SetFilter("All Files (*.*)|*.*|");
  FileData.DefExt = "txt";

  // INSERT>> Your constructor code here.
}


TTriangleApp::~TTriangleApp()
{
  // INSERT>> Your destructor code here.
}


//--------------------------------------------------------
// TTriangleApp
// ~~~~~
// Application intialization.
//
void TTriangleApp::InitMainWindow()
{
  if (nCmdShow != SW_HIDE)
    nCmdShow = (nCmdShow != SW_SHOWMINNOACTIVE) ? SW_SHOWNORMAL : nCmdShow;

  TSDIDecFrame* frame = new TSDIDecFrame(0, GetName(), 0, false);

  // Assign ICON w/ this application.
  //
  frame->SetIcon(this, IDI_SDIAPPLICATION);
  frame->SetIconSm(this, IDI_SDIAPPLICATION);

  // Menu associated with window and accelerator table associated with table.
  //
  frame->AssignMenu(IDM_SDI);

  // Associate with the accelerator table.
  //
  frame->Attr.AccelTable = IDM_SDI;

  SetMainWindow(frame);

  frame->SetMenuDescr(TMenuDescr(IDM_SDI));
}



//--------------------------------------------------------
// TTriangleApp
// ~~~~~~~~~~~
// Menu File New command
//
void TTriangleApp::CmFileNew()
{
}


//--------------------------------------------------------
// TTriangleApp
// ~~~~~~~~~~~
// Menu File Open command
//
void TTriangleApp::CmFileOpen()
{
  // Display standard Open dialog box to select a file name.
  //
  *FileData.FileName = 0;

  TTriangleWindow* client = TYPESAFE_DOWNCAST(GetMainWindow()->GetClientWindow(), TTriangleWindow);     // Client window for the frame.
  if (client->CanClose())
    if (TFileOpenDialog(GetMainWindow(), FileData).Execute() == IDOK)
      OpenFile();
}


void TTriangleApp::OpenFile(const char* fileName)
{
  if (fileName)
    strcpy(FileData.FileName, fileName);

}


//{{TSDIDecFrame Implementation}}


TSDIDecFrame::TSDIDecFrame(TWindow* parent, const char far* title, TWindow* clientWnd, bool trackMenuSelection, TModule* module)
:
  TDecoratedFrame(parent, title, !clientWnd ? new TTriangleWindow(0, "") : clientWnd, trackMenuSelection, module)
{
  // INSERT>> Your constructor code here.

}


TSDIDecFrame::~TSDIDecFrame()
{
  // INSERT>> Your destructor code here.

}


//--------------------------------------------------------
// TTriangleApp
// ~~~~~~~~~~~
// Menu Help About Triangle command
//
void TTriangleApp::CmHelpAbout()
{
  // Show the modal dialog.
  //
  TTriangleAboutDlg(MainWindow).Execute();
}



int OwlMain(int , char* [])
{
  TTriangleApp   app;
  return app.Run();
}
