//----------------------------------------------------------------------------
//  Project RGBTrngl
//  Waite Group Press
//  Copyright  1996. All Rights Reserved.
//
//  SUBSYSTEM:    RGBTrngl Application
//  FILE:         rgbtrnga.cpp
//  AUTHOR:       Richard S. Wright Jr.
//
//  OVERVIEW
//  ~~~~~~~~
//  Source file for implementation of TRGBTrnglApp (TApplication).
//
//----------------------------------------------------------------------------

#include <owl/pch.h>

#include <stdio.h>

#include "rgbtrnga.h"
#include "rgbtrngw.h"                        // Definition of client class.
#include "rgbtrnad.h"                        // Definition of about dialog.



//{{TRGBTrnglApp Implementation}}


//
// Build a response table for all messages/commands handled
// by the application.
//
DEFINE_RESPONSE_TABLE1(TRGBTrnglApp, TApplication)
//{{TRGBTrnglAppRSP_TBL_BEGIN}}
  EV_COMMAND(CM_FILENEW, CmFileNew),
  EV_COMMAND(CM_FILEOPEN, CmFileOpen),
  EV_COMMAND(CM_HELPABOUT, CmHelpAbout),
//{{TRGBTrnglAppRSP_TBL_END}}
END_RESPONSE_TABLE;


//--------------------------------------------------------
// TRGBTrnglApp
//
TRGBTrnglApp::TRGBTrnglApp() : TApplication("RGB Triangle")
{

  // Common file file flags and filters for Open/Save As dialogs.  Filename and directory are
  // computed in the member functions CmFileOpen, and CmFileSaveAs.
  //
  FileData.Flags = OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT;
  FileData.SetFilter("All Files (*.*)|*.*|");
  FileData.DefExt = "txt";

  // INSERT>> Your constructor code here.
}


TRGBTrnglApp::~TRGBTrnglApp()
{
  // INSERT>> Your destructor code here.
}


//--------------------------------------------------------
// TRGBTrnglApp
// ~~~~~
// Application intialization.
//
void TRGBTrnglApp::InitMainWindow()
{
  if (nCmdShow != SW_HIDE)
    nCmdShow = (nCmdShow != SW_SHOWMINNOACTIVE) ? SW_SHOWNORMAL : nCmdShow;

  TSDIDecFrame* frame = new TSDIDecFrame(0, GetName(), 0, false);

  // Assign ICON w/ this application.
  //
  frame->SetIcon(this, IDI_SDIAPPLICATION);
  frame->SetIconSm(this, IDI_SDIAPPLICATION);

  // Menu associated with window and accelerator table associated with table.
  //
  frame->AssignMenu(IDM_SDI);

  // Associate with the accelerator table.
  //
  frame->Attr.AccelTable = IDM_SDI;

  SetMainWindow(frame);

  frame->SetMenuDescr(TMenuDescr(IDM_SDI));
}



//--------------------------------------------------------
// TRGBTrnglApp
// ~~~~~~~~~~~
// Menu File New command
//
void TRGBTrnglApp::CmFileNew()
{
}


//--------------------------------------------------------
// TRGBTrnglApp
// ~~~~~~~~~~~
// Menu File Open command
//
void TRGBTrnglApp::CmFileOpen()
{
  // Display standard Open dialog box to select a file name.
  //
  *FileData.FileName = 0;

  TRGBTrnglWindow* client = TYPESAFE_DOWNCAST(GetMainWindow()->GetClientWindow(), TRGBTrnglWindow);     // Client window for the frame.
  if (client->CanClose())
    if (TFileOpenDialog(GetMainWindow(), FileData).Execute() == IDOK)
      OpenFile();
}


void TRGBTrnglApp::OpenFile(const char* fileName)
{
  if (fileName)
    strcpy(FileData.FileName, fileName);

}


//{{TSDIDecFrame Implementation}}


TSDIDecFrame::TSDIDecFrame(TWindow* parent, const char far* title, TWindow* clientWnd, bool trackMenuSelection, TModule* module)
:
  TDecoratedFrame(parent, title, !clientWnd ? new TRGBTrnglWindow(0, "") : clientWnd, trackMenuSelection, module)
{
  // INSERT>> Your constructor code here.

}


TSDIDecFrame::~TSDIDecFrame()
{
  // INSERT>> Your destructor code here.

}


//--------------------------------------------------------
// TRGBTrnglApp
// ~~~~~~~~~~~
// Menu Help About RGBTrngl command
//
void TRGBTrnglApp::CmHelpAbout()
{
  // Show the modal dialog.
  //
  TRGBTrnglAboutDlg(MainWindow).Execute();
}



int OwlMain(int , char* [])
{
  TRGBTrnglApp   app;
  return app.Run();
}
