//----------------------------------------------------------------------------
//  Project Shadow
//  Waite Group Press
//  Copyright  1996. All Rights Reserved.
//
//  SUBSYSTEM:    Shadow Application
//  FILE:         shadwapp.cpp
//  AUTHOR:       Richard S. Wright Jr.
//
//  OVERVIEW
//  ~~~~~~~~
//  Source file for implementation of TShadowApp (TApplication).
//
//----------------------------------------------------------------------------

#include <owl/pch.h>

#include <stdio.h>

#include "shadwapp.h"
#include "shdwwndw.h"                        // Definition of client class.
#include "shdwabtd.h"                        // Definition of about dialog.


//{{TShadowApp Implementation}}


//
// Build a response table for all messages/commands handled
// by the application.
//
DEFINE_RESPONSE_TABLE1(TShadowApp, TApplication)
//{{TShadowAppRSP_TBL_BEGIN}}
  EV_COMMAND(CM_HELPABOUT, CmHelpAbout),
//{{TShadowAppRSP_TBL_END}}
END_RESPONSE_TABLE;


//--------------------------------------------------------
// TShadowApp
//
TShadowApp::TShadowApp() : TApplication("Shadow Demo")
{

  // Common file file flags and filters for Open/Save As dialogs.  Filename and directory are
  // computed in the member functions CmFileOpen, and CmFileSaveAs.
  //
  FileData.Flags = OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT;
  FileData.SetFilter("All Files (*.*)|*.*|");
  FileData.DefExt = "txt";

  // INSERT>> Your constructor code here.
}


TShadowApp::~TShadowApp()
{
  // INSERT>> Your destructor code here.
}


//--------------------------------------------------------
// TShadowApp
// ~~~~~
// Application intialization.
//
void TShadowApp::InitMainWindow()
{
  if (nCmdShow != SW_HIDE)
    nCmdShow = (nCmdShow != SW_SHOWMINNOACTIVE) ? SW_SHOWNORMAL : nCmdShow;

  TSDIDecFrame* frame = new TSDIDecFrame(0, GetName(), 0, false);

  // Assign ICON w/ this application.
  //
  frame->SetIcon(this, IDI_SDIAPPLICATION);
  frame->SetIconSm(this, IDI_SDIAPPLICATION);

  // Menu associated with window and accelerator table associated with table.
  //
  frame->AssignMenu(IDM_SDI);

  // Associate with the accelerator table.
  //
  frame->Attr.AccelTable = IDM_SDI;

  SetMainWindow(frame);

  frame->SetMenuDescr(TMenuDescr(IDM_SDI));
}



//--------------------------------------------------------
// TShadowApp
// ~~~~~~~~~~~
// Menu File New command
//
void TShadowApp::CmFileNew()
{
}


//--------------------------------------------------------
// TShadowApp
// ~~~~~~~~~~~
// Menu File Open command
//
void TShadowApp::CmFileOpen()
{
  // Display standard Open dialog box to select a file name.
  //
  *FileData.FileName = 0;

  TShadowWindow* client = TYPESAFE_DOWNCAST(GetMainWindow()->GetClientWindow(), TShadowWindow);     // Client window for the frame.
  if (client->CanClose())
    if (TFileOpenDialog(GetMainWindow(), FileData).Execute() == IDOK)
      OpenFile();
}


void TShadowApp::OpenFile(const char* fileName)
{
  if (fileName)
    strcpy(FileData.FileName, fileName);

}


//{{TSDIDecFrame Implementation}}


TSDIDecFrame::TSDIDecFrame(TWindow* parent, const char far* title, TWindow* clientWnd, bool trackMenuSelection, TModule* module)
:
  TDecoratedFrame(parent, title, !clientWnd ? new TShadowWindow(0, "") : clientWnd, trackMenuSelection, module)
{
  // INSERT>> Your constructor code here.

}


TSDIDecFrame::~TSDIDecFrame()
{
  // INSERT>> Your destructor code here.

}


//--------------------------------------------------------
// TShadowApp
// ~~~~~~~~~~~
// Menu Help About Shadow command
//
void TShadowApp::CmHelpAbout()
{
  // Show the modal dialog.
  //
  TShadowAboutDlg(MainWindow).Execute();
}



int OwlMain(int , char* [])
{
  TShadowApp   app;
  return app.Run();
}
